@@ -2,7 +2,7 @@
   **
   ** conf.c
   **
-  ** I originaly wrote conf.c as part of the drpoxy package
+  ** I originally wrote conf.c as part of the drpoxy package
   ** thanks to Matthew Pratt and others for their additions.
   **
   ** Copyright 1999 Jeroen Vreeken (pe1rxq@chello.nl)
@@ -25,14 +25,13 @@
  */
 #include "motion.h"
 
+
 #ifdef __freebsd__
 #include "video_freebsd.h"
 #else
 #include "video.h"
 #endif /* __freebsd__ */
 
-#include "conf.h"
-#include "track.h"
 
 #define stripnewline(x) {if ((x)[strlen(x)-1]=='\n') (x)[strlen(x) - 1] = 0; }
 
@@ -93,7 +92,7 @@ struct config conf_template = {
 	timelapse:             0,
 	timelapse_mode:        DEF_TIMELAPSE_MODE,
 #ifdef  __freebsd__
-	tuner_device:          TUNER_DEVICE,
+	tuner_device:          NULL,
 #endif
 	video_device:          VIDEO_DEVICE,
 	vidpipe:               NULL,
@@ -110,6 +109,7 @@ struct config conf_template = {
 	sql_log_snapshot:      1,
 	sql_log_mpeg:          0,
 	sql_log_timelapse:     0,
+	sql_query:             DEF_SQL_QUERY,
 	mysql_db:              NULL,
 	mysql_host:            NULL,
 	mysql_user:            NULL,
@@ -130,6 +130,7 @@ struct config conf_template = {
 	text_changes:          0,
 	text_left:             NULL,
 	text_right:            DEF_TIMESTAMP,
+	text_event:            DEF_EVENTSTAMP,
 	text_double:           0,
 	despeckle:             NULL,
 	minimum_motion_frames: 1,
@@ -138,14 +139,14 @@ struct config conf_template = {
 
 
 
-static struct context **copy_bool(struct context **, char *, int);
-static struct context **copy_int(struct context **, char *, int);
-static struct context **config_thread(struct context **cnt, char *str, int val);
+static struct context **copy_bool(struct context **, const char *, int);
+static struct context **copy_int(struct context **, const char *, int);
+static struct context **config_thread(struct context **cnt, const char *str, int val);
 
-static char *print_bool(struct context **, char **, int, int);
-static char *print_int(struct context **, char **, int, int);
-static char *print_string(struct context **, char **, int, int);
-static char *print_thread(struct context **, char **, int, int);
+static const char *print_bool(struct context **, char **, int, int);
+static const char *print_int(struct context **, char **, int, int);
+static const char *print_string(struct context **, char **, int, int);
+static const char *print_thread(struct context **, char **, int, int);
 
 static void usage(void);
 
@@ -339,7 +340,7 @@ config_param config_params[] = {
 	},
 	{
 	"switchfilter",
-	"# Try to filter out noise generated by roundrobin (default: 0)",
+	"# Try to filter out noise generated by roundrobin (default: off)",
 	CONF_OFFSET(switchfilter),
 	copy_bool,
 	print_bool
@@ -529,7 +530,10 @@ config_param config_params[] = {
 	{
 	"ffmpeg_cap_new",
 	"\n############################################################\n"
-	"# Film (mpeg) File Output - ffmpeg based\n"
+	"# FFMPEG related options\n"
+	"# Film (mpeg) file output, and deinterlacing of the video input\n"
+	"# The options movie_filename and timelapse_filename are also used\n"
+	"# by the ffmpeg feature\n"
 	"############################################################\n\n"
 	"# Use ffmpeg to encode mpeg movies in realtime (default: off)",
 	CONF_OFFSET(ffmpeg_cap_new),
@@ -591,6 +595,15 @@ config_param config_params[] = {
 	copy_string,
 	print_string
 	},
+	{
+	"ffmpeg_deinterlace",
+	"# Use ffmpeg to deinterlace video. Necessary if you use an analog camera\n"
+	"# and see horizontal combing on moving objects in video or pictures.\n"
+	"# (default: off)",
+	CONF_OFFSET(ffmpeg_deinterlace),
+	copy_bool,
+	print_bool
+	},
 #endif /* HAVE_FFMPEG */
 
 	{
@@ -610,10 +623,11 @@ config_param config_params[] = {
 	"# Text Display\n"
 	"# %Y = year, %m = month, %d = date,\n"
 	"# %H = hour, %M = minute, %S = second, %T = HH:MM:SS,\n"
-	"# %v = event, %q = frame number, \\n = new line,\n"
-	"# %D = changed pixels, %N = noise level,\n"
+	"# %v = event, %q = frame number, %t = thread (camera) number,\n"
+	"# %D = changed pixels, %N = noise level, \\n = new line,\n"
 	"# %i and %J = width and height of motion area,\n"
 	"# %K and %L = X and Y coordinates of motion center\n"
+	"# %C = value defined by text_event - do not use with text_event!\n"
 	"# You can put quotation marks around the text to allow\n"
 	"# leading spaces\n"
 	"############################################################\n\n"
@@ -652,6 +666,18 @@ config_param config_params[] = {
 	print_bool
 	},
 	{
+	"text_event",
+	"# This option defines the value of the special event conversion specifier %C\n"
+	"# You can use any conversion specifier in this option except %C. Date and time\n"
+	"# values are from the timestamp of the first image in the current event.\n"
+	"# Default: %Y%m%d%H%M%S\n"
+	"# The idea is that %C can be used filenames and text_left/right for creating\n"
+	"# a unique identifier for each event.",
+	CONF_OFFSET(text_event),
+	copy_string,
+	print_string
+	},
+	{
 	"text_double",
 	"# Draw characters at twice normal size on images. (default: off)",
 	CONF_OFFSET(text_double),
@@ -667,14 +693,15 @@ config_param config_params[] = {
 	"# you can use conversion specifiers\n"
 	"# %Y = year, %m = month, %d = date,\n"
 	"# %H = hour, %M = minute, %S = second,\n"
-	"# %v = event, %q = frame number\n"
+	"# %v = event, %q = frame number, %t = thread (camera) number,\n"
 	"# %D = changed pixels, %N = noise level,\n"
 	"# %i and %J = width and height of motion area,\n"
 	"# %K and %L = X and Y coordinates of motion center\n"
+	"# %C = value defined by text_event\n"
 	"# Quotation marks round string are allowed.\n"
 	"############################################################\n\n"
 	"# Target base directory for pictures and films\n"
-	"# Recommended to use absolute patch. (Default: current working directory)",
+	"# Recommended to use absolute path. (Default: current working directory)",
 	CONF_OFFSET(filepath),
 	copy_string,
 	print_string
@@ -707,12 +734,13 @@ config_param config_params[] = {
 	},
 #ifdef HAVE_FFMPEG
 	{
-	"ffmpeg_filename",
+	"movie_filename",
 	"# File path for motion triggered ffmpeg films (mpeg) relative to target_dir\n"
 	"# Default: "DEF_MPEGPATH"\n"
 	"# Default value is equivalent to legacy oldlayout option\n"
 	"# For Motion 3.0 compatible mode choose: %Y/%m/%d/%H%M%S\n"
-	"# File extension .mpg or .avi is automatically added so do not include this",
+	"# File extension .mpg or .avi is automatically added so do not include this\n"
+	"# This option was previously called ffmpeg_filename",
 	CONF_OFFSET(mpegpath),
 	copy_string,
 	print_string
@@ -742,7 +770,7 @@ config_param config_params[] = {
 	},
 	{
 	"webcam_quality",
-	"# Quality of the jpeg images produced (default: 30)",
+	"# Quality of the jpeg images produced (default: 50)",
 	CONF_OFFSET(webcam_quality),
 	copy_int,
 	print_int
@@ -817,12 +845,19 @@ config_param config_params[] = {
 	"############################################################\n\n"
 	"# Type of tracker (0=none (default), 1=stepper, 2=iomojo, 3=pwc, 4=generic)\n"
 	"# The generic type enables the definition of motion center and motion size to\n"
-	"# be used with the convertion specifiers for options like on_motion_detected",
+	"# be used with the conversion specifiers for options like on_motion_detected",
 	TRACK_OFFSET(type),
 	copy_int,
 	print_int
 	},
 	{
+	"track_auto",
+	"# Enable auto tracking (default: off)",
+	TRACK_OFFSET(active),
+	copy_bool,
+	print_bool
+	},
+	{
 	"track_port",
 	"# Serial port of motor (default: none)",
 	TRACK_OFFSET(port),
@@ -837,11 +872,25 @@ config_param config_params[] = {
 	print_int
 	},
 	{
+	"track_motory",
+	"# Motor number for y-axis (default: -1)",
+	TRACK_OFFSET(motory),
+	copy_int,
+	print_int
+	},
+	{
 	"track_maxx",
 	"# Maximum value on x-axis (default: 0)",
 	TRACK_OFFSET(maxx),
 	copy_int,
 	print_int
+	},
+	 {
+	"track_maxy",
+	"# Maximum value on y-axis (default: 0)",
+	TRACK_OFFSET(maxy),
+	copy_int,
+	print_int
 	},
 	{
 	"track_iomojo_id",
@@ -851,15 +900,41 @@ config_param config_params[] = {
 	print_int
 	},
 	{
+	"track_step_angle_x",
+	"# Angle in degrees the camera moves per step on the X-axis\n"
+	"# with auto-track (default: 10)\n"
+	"# Currently only used with pwc type cameras",
+	TRACK_OFFSET(step_angle_x),
+	copy_int,
+	print_int
+	},
+	{
+	"track_step_angle_y",
+	"# Angle in degrees the camera moves per step on the Y-axis\n"
+	"# with auto-track (default: 10)\n"
+	"# Currently only used with pwc type cameras",
+	TRACK_OFFSET(step_angle_y),
+	copy_int,
+	print_int
+	},
+	{
+	"track_move_wait",
+	"# Delay to wait for after tracking movement as number\n"
+	"# of picture frames (default: 10)",
+	TRACK_OFFSET(move_wait),
+	copy_int,
+	print_int
+	},
+	{
 	"track_speed",
-	"# Speed to set the motor to (default: 255)",
+	"# Speed to set the motor to (stepper motor option) (default: 255)",
 	TRACK_OFFSET(speed),
 	copy_int,
 	print_int
 	},
 	{
 	"track_stepsize",
-	"# Number of steps to make (default: 40)",
+	"# Number of steps to make (stepper motor option) (default: 40)",
 	TRACK_OFFSET(stepsize),
 	copy_int,
 	print_int
@@ -872,10 +947,15 @@ config_param config_params[] = {
 	"# You can use conversion specifiers for the on_xxxx commands\n"
 	"# %Y = year, %m = month, %d = date,\n"
 	"# %H = hour, %M = minute, %S = second,\n"
-	"# %v = event, %q = frame number\n"
+	"# %v = event, %q = frame number, %t = thread (camera) number,\n"
 	"# %D = changed pixels, %N = noise level,\n"
 	"# %i and %J = width and height of motion area,\n"
 	"# %K and %L = X and Y coordinates of motion center\n"
+	"# %C = value defined by text_event\n"
+	"# %f = filename with full path\n"
+	"# %n = number indicating filetype\n"
+	"# Both %f and %n are only defined for on_picture_save,\n"
+	"# on_movie_start and on_movie_end\n" 
 	"# Quotation marks round string are allowed.\n"
 	"############################################################\n\n"
 	"# Do not sound beeps when detecting motion (default: on)\n"
@@ -903,7 +983,7 @@ config_param config_params[] = {
 	{
 	"on_picture_save",
 	"# Command to be executed when a picture (.ppm|.jpg) is saved (default: none)\n"
-	"# The filename of the picture is appended as an argument for the command.",
+	"# To give the filename as an argument to a command append it with %f",
 	CONF_OFFSET(on_picture_save),
 	copy_string,
 	print_string
@@ -919,7 +999,7 @@ config_param config_params[] = {
 	{
 	"on_movie_start",
 	"# Command to be executed when a movie file (.mpg|.avi) is created. (default: none)\n"
-	"# Filename of movie is appended as an argument for the command.",
+	"# To give the filename as an argument to a command append it with %f",
 	CONF_OFFSET(on_movie_start),
 	copy_string,
 	print_string
@@ -927,7 +1007,7 @@ config_param config_params[] = {
 	{
 	"on_movie_end",
 	"# Command to be executed when a movie file (.mpg|.avi) is closed. (default: none)\n"
-	"# Filename of movie is appended as an argument for the command.",
+	"# To give the filename as an argument to a command append it with %f",
 	CONF_OFFSET(on_movie_end),
 	copy_string,
 	print_string
@@ -967,6 +1047,19 @@ config_param config_params[] = {
 	copy_bool,
 	print_bool
 	},
+	{
+	"sql_query",
+	"# SQL query string that is sent to the database\n"
+	"# Use same conversion specifiers has for text features\n"
+	"# Additional special conversion specifiers are\n"
+	"# %n = the number representing the file_type\n"
+	"# %f = filename with full path\n"
+	"# Default value:\n"
+	"# insert into security(camera, filename, frame, file_type, time_stamp, text_event) values('%t', '%f', '%q', '%n', '%Y-%m-%d %T', '%C')",
+	CONF_OFFSET(sql_query),
+	copy_string,
+	print_string
+	},
 
 #endif /* defined(HAVE_MYSQL) || defined(HAVE_PGSQL) */
 
@@ -1130,12 +1223,12 @@ static void conf_cmdline (struct context *cnt, int thread)
 
 
 /* conf_cmdparse sets a config option given by 'cmd' to the value given by 'arg1'.
- * Based on the name of the option it searces through the struct 'config_params'
+ * Based on the name of the option it searches through the struct 'config_params'
  * for an option where the config_params[i].param_name matches the option.
  * By calling the function pointed to by config_params[i].copy the option gets
  * assigned.
  */
-struct context **conf_cmdparse(struct context **cnt, char *cmd, char *arg1)
+struct context **conf_cmdparse(struct context **cnt, const char *cmd, const char *arg1)
 {
 	int i = 0;
 
@@ -1147,8 +1240,13 @@ struct context **conf_cmdparse(struct context **cnt, char *cmd, char *arg1)
 	 */
 	while( config_params[i].param_name != NULL ) {
 		if(!strncasecmp(cmd, config_params[i].param_name , 255 + 50)) { // Why +50?
-			if(config_params[i].conf_value && !arg1)
-				return cnt;
+	
+			/* if config_param is string we don't want to check arg1 */		
+			if (strcmp(config_type(&config_params[i]),"string")) {
+				if(config_params[i].conf_value && !arg1)
+					return cnt;
+			}
+			
 			/* We call the function given by the pointer config_params[i].copy
 			 * If the option is a bool, copy_bool is called.
 			 * If the option is an int, copy_int is called.
@@ -1167,7 +1265,7 @@ struct context **conf_cmdparse(struct context **cnt, char *cmd, char *arg1)
 	}
 
 	/* We reached the end of config_params without finding a matching option */
-	motion_log(cnt[0], LOG_ERR, 1, "Unknown config option \"%s\"",cmd);
+	motion_log(LOG_ERR, 0, "Unknown config option \"%s\"",cmd);
 
 	return cnt;
 }
@@ -1239,7 +1337,7 @@ static struct context **conf_process(struct context **cnt, FILE *fp)
 			}
 			/* else arg1 stays null pointer */
 
-			cnt=conf_cmdparse(cnt, cmd, arg1);
+			cnt = conf_cmdparse(cnt, cmd, arg1);
 		}
 	}
 
@@ -1252,30 +1350,32 @@ static struct context **conf_process(struct context **cnt, FILE *fp)
  */
 void conf_print(struct context **cnt)
 {
+	const char *retval;
 	char *val;
 	int i, thread;
 	FILE *conffile;
 
 	for (thread=0; cnt[thread]; thread++) {
-		motion_log(cnt[0], LOG_INFO, 1, "Writing config file to %s",cnt[thread]->conf_filename);
-		conffile=myfopen(cnt[0] ,cnt[thread]->conf_filename, "w");
+		motion_log(LOG_INFO, 1, "Writing config file to %s",cnt[thread]->conf_filename);
+		conffile=myfopen(cnt[thread]->conf_filename, "w");
 		if (!conffile)
 			continue;
 		fprintf(conffile, "# %s\n", cnt[thread]->conf_filename);
 		fprintf(conffile, "#\n# This config file was generated by motion " VERSION "\n");
 		fprintf(conffile, "\n\n");
 		for (i=0; config_params[i].param_name; i++) {
-			val=config_params[i].print(cnt, NULL, i, thread);
+			retval=config_params[i].print(cnt, NULL, i, thread);
 			/*If config parameter has a value (not NULL) print it to the config file*/
-			if (val) {
+			if (retval) {
 				fprintf(conffile, "%s\n", config_params[i].param_help);
 				/* If the option is a text_* and first char is a space put
 				   quotation marks around to allow leading spaces */
-				if (strncmp(config_params[i].param_name, "text", 4) || strncmp(val, " ", 1))
-					fprintf(conffile, "%s %s\n\n", config_params[i].param_name, val);
+				if (strncmp(config_params[i].param_name, "text", 4) || strncmp(retval, " ", 1))
+					fprintf(conffile, "%s %s\n\n", config_params[i].param_name, retval);
 				else
-					fprintf(conffile, "%s \"%s\"\n\n", config_params[i].param_name, val);
+					fprintf(conffile, "%s \"%s\"\n\n", config_params[i].param_name, retval);
 			} else {
+				val = NULL;
 				config_params[i].print(cnt, &val, i, thread);
 				/* It can either be a thread file parameter or a disabled parameter
 				   If it is a thread parameter write it out
@@ -1311,7 +1411,7 @@ void conf_print(struct context **cnt)
  * - All config (cnt.conf) struct members pointing to a string are changed
  *   so that they point to a malloc'ed piece of memory containing a copy of
  *   the string given in conf_template.
- * - motion.conf is opened processed. The process populates the cnt[0] and
+ * - motion.conf is opened and processed. The process populates the cnt[0] and
  *   for each thread config file it populates a cnt[1], cnt[2]... for each
  *   thread
  * - Finally it process the options given in the command line. This is done
@@ -1324,8 +1424,8 @@ struct context ** conf_load (struct context **cnt)
 	char filename[PATH_MAX];
 	int i;
 	/* We preserve argc and argv because they get overwritten by the memcpy command */
-	char **argv=cnt[0]->conf.argv;
-	int argc=cnt[0]->conf.argc;
+	char **argv = cnt[0]->conf.argv;
+	int argc = cnt[0]->conf.argc;
 
 	/* Copy the template config structure with all the default config values
 	 * into cnt[0]->conf
@@ -1338,13 +1438,13 @@ struct context ** conf_load (struct context **cnt)
 	 * 2. Copy the conf_template given string to the reserved memory
 	 * 3. Change the cnt[0] member (char*) pointing to the string in reserved memory
 	 * This ensures that we can free and malloc the string when changed
-	 * via xmlrpc or config file or command line options
+	 * via http remote control or config file or command line options
 	 */
 	malloc_strings(cnt[0]);
 
 	/* Restore the argc and argv */
-	cnt[0]->conf.argv=argv;
-	cnt[0]->conf.argc=argc;
+	cnt[0]->conf.argv = argv;
+	cnt[0]->conf.argc = argc;
 
 	/* Open the motion.conf file. We try in this sequence:
 	 * 1. commandline
@@ -1353,7 +1453,7 @@ struct context ** conf_load (struct context **cnt)
 	 * 4. sysconfig/motion.conf
 	 */
 	/* Get filename from commandline */
-	cnt[0]->conf_filename[0]=0;
+	cnt[0]->conf_filename[0] = 0;
 	conf_cmdline(cnt[0], -1);
 	if (cnt[0]->conf_filename[0]){ /* User has supplied filename on commandline*/
 		strcpy(filename, cnt[0]->conf_filename);
@@ -1361,7 +1461,7 @@ struct context ** conf_load (struct context **cnt)
 	}
 	if (!fp){      /* Commandline didn't work, try current dir */
 		if (cnt[0]->conf_filename[0])
-			motion_log(cnt[0], -1, 1, "Configfile %s not found - trying defaults.", filename);
+			motion_log(-1, 1, "Configfile %s not found - trying defaults.", filename);
 		sprintf(filename, "motion.conf");
 		fp = fopen (filename, "r");
 	}
@@ -1372,20 +1472,20 @@ struct context ** conf_load (struct context **cnt)
 			sprintf(filename, "%s/motion.conf", sysconfdir);
 			fp = fopen (filename, "r");
 			if (!fp)        /* there is no config file.... use defaults */
-				motion_log(cnt[0], -1, 1, "could not open configfile %s",filename);
+				motion_log(-1, 1, "could not open configfile %s",filename);
 		}
 	}
 
 	/* Now we process the motion.conf config file and close it*/
 	if (fp) {
 		strcpy(cnt[0]->conf_filename, filename);
-		motion_log(cnt[0], LOG_INFO, 0, "Processing thread 0 - config file %s",filename);
+		motion_log(LOG_INFO, 0, "Processing thread 0 - config file %s",filename);
 		cnt=conf_process(cnt, fp);
 		fclose(fp);
 	}
 	
 
-	/* For each thread (given by cnt[i] being not null
+	/* For each thread (given by cnt[i]) being not null
 	 * cnt is an array of pointers to a context type structure
 	 * cnt[0] is the default context structure
 	 * cnt[1], cnt[2], ... are context structures for each thread
@@ -1411,18 +1511,18 @@ struct context ** conf_load (struct context **cnt)
 void malloc_strings (struct context * cnt)
 {
 	int i = 0;
-	void *val;
+	char **val;
 	while( config_params[i].param_name != NULL ) {
 		if (config_params[i].copy == copy_string) { /* if member is a string */
 			/* val is made to point to a pointer to the current string */
-			val=(void *)cnt+config_params[i].conf_value;
+			val = (char **)((char *)cnt+config_params[i].conf_value);
 
 			/* if there is a string, malloc() space for it, copy
 			 * the string to new space, and point to the new
 			 * string. we don't free() because we're copying a
 			 * static string.
 			 */
-			*(char **)val = mystrdup(*(char **)val);
+			*val = mystrdup(*val);
 		}
 		i++;
 	}
@@ -1452,18 +1552,18 @@ void malloc_strings (struct context * cnt)
  ***********************************************************************/
 
 /* copy_bool assigns a config option to a new boolean value.
- * The boolean is given as a string in str which is coverted to 0 or 1 
+ * The boolean is given as a string in str which is converted to 0 or 1 
  * by the function. Values 1, yes and on are converted to 1 ignoring case.
  * Any other value is converted to 0.
  */
-static struct context **copy_bool (struct context **cnt, char *str, int val_ptr)
+static struct context **copy_bool (struct context **cnt, const char *str, int val_ptr)
 {
 	void *tmp;
 	int i;
 
 	i=-1;
 	while(cnt[++i]) {
-		tmp=(void *)cnt[i]+(int)val_ptr;
+		tmp = (char *)cnt[i]+(int)val_ptr;
 		if ( !strcmp(str, "1") || !strcasecmp(str, "yes") || !strcasecmp(str,"on")) {
 			*((int *)tmp) = 1;
 		} else {
@@ -1475,17 +1575,17 @@ static struct context **copy_bool (struct context **cnt, char *str, int val_ptr)
 	return cnt;
 }
 
-/* copy_int assigns a config option to a new interger value.
- * The integer is given as a string in str which is coverted to integer by the function.
+/* copy_int assigns a config option to a new integer value.
+ * The integer is given as a string in str which is converted to integer by the function.
  */
-static struct context ** copy_int(struct context **cnt, char *str, int val_ptr)
+static struct context ** copy_int(struct context **cnt, const char *str, int val_ptr)
 {
 	void *tmp;
 	int i;
 
 	i=-1;
 	while(cnt[++i]) {
-		tmp=(void *)cnt[i]+val_ptr;
+		tmp = (char *)cnt[i]+val_ptr;
 		*((int *)tmp) = atoi(str);
 		if (cnt[0]->threadnr)
 			return cnt;
@@ -1493,26 +1593,26 @@ static struct context ** copy_int(struct context **cnt, char *str, int val_ptr)
 	return cnt;
 }
 
-/* copy_string assigns a config option to a new interger value.
+/* copy_string assigns a new string value to a config option.
  * Strings are handled differently from bool and int.
  * the char *conf->option that we are working on is free()'d
  * (if memory for it has already been malloc()'d), and set to
  * a freshly malloc()'d string with the value from str,
  * or NULL if str is blank
  */
-struct context **copy_string(struct context **cnt, char *str, int val_ptr)
+struct context **copy_string(struct context **cnt, const char *str, int val_ptr)
 {
-	void *tmp;
+	char **tmp;
 	int i;
 
 	i=-1;
 	while(cnt[++i]) {
-		tmp=(void *)cnt[i]+val_ptr;
+		tmp = (char **)((char *)cnt[i] + val_ptr);
 
 		/* mystrcpy assigns the new string value
 		 * including free'ing and reserving new memory for it.
 		 */
-		*(char **)tmp = mystrcpy(*(char **)tmp, str);
+		*tmp = mystrcpy(*tmp, str);
 
 		/* set the option on all threads if setting the option
 		 * for thread 0; otherwise just set that one thread's option
@@ -1533,13 +1633,13 @@ struct context **copy_string(struct context **cnt, char *str, int val_ptr)
  *
  * from - pointer to the new string we want to copy
  * to   - the pointer to the current string (or pointing to NULL)
- *        If not NULL the memory it points to is fre'ed.
+ *        If not NULL the memory it points to is free'd.
  * function returns pointer to the new string which is in malloc'ed memory
  * FIXME The strings that are malloc'ed with this function should be freed
  * when the motion program is terminated normally instead of relying on the
  * OS to clean up.
  */
-char *mystrcpy(char *to, char *from)
+char *mystrcpy(char *to, const char *from)
 {
 	/* free the memory used by the to string, if such memory exists,
 	 * and return a pointer to a freshly malloc()'d string with the
@@ -1560,7 +1660,7 @@ char *mystrcpy(char *to, char *from)
  * variable PATH_MAX to ensure that config options can always contain
  * a really long path but no more than that.
  */
-char *mystrdup(char *from)
+char *mystrdup(const char *from)
 {
 	char *tmp;
 	int stringlength;
@@ -1570,7 +1670,7 @@ char *mystrdup(char *from)
 	} else {
 		stringlength = strlen(from);
 		stringlength = (stringlength < PATH_MAX ? stringlength : PATH_MAX);
-		tmp = (char *)mymalloc(NULL, stringlength + 1);
+		tmp = (char *)mymalloc(stringlength + 1);
 		strncpy(tmp, from, stringlength);
 
 		/* We must ensure the string always has a NULL terminator.
@@ -1584,27 +1684,27 @@ char *mystrdup(char *from)
 	return tmp;
 }
 
-char *config_type(config_param *configparam)
+const char *config_type(config_param *configparam)
 {
-	if (configparam->copy==copy_string)
+	if (configparam->copy == copy_string)
 		return "string";
-	if (configparam->copy==copy_int)
+	if (configparam->copy == copy_int)
 		return "int";
-	if (configparam->copy==copy_bool)
+	if (configparam->copy == copy_bool)
 		return "bool";
 	return "unknown";
 }
 
-static char *print_bool(struct context **cnt, char **str, int parm, int threadnr)
+static const char *print_bool(struct context **cnt, char **str ATTRIBUTE_UNUSED,
+                              int parm, int threadnr)
 {
 	int val=config_params[parm].conf_value;
 
 	if (threadnr &&
-	    *(int*)((void*)cnt[threadnr]+(int)val)==
-	    *(int*)((void*)cnt[0]+(int)val))
+	    *(int*)((char *)cnt[threadnr] + val) == *(int*)((char *)cnt[0] + val))
 		return NULL;
 
-	if (*(int*)((void*)cnt[threadnr]+(int)val))
+	if (*(int*)((char *)cnt[threadnr] + val))
 		return "on";
 	else
 		return "off";
@@ -1616,48 +1716,53 @@ static char *print_bool(struct context **cnt, char **str, int parm, int threadnr
  * option in thread 0. If the value is the same, NULL is returned which means that
  * the option is not written to the thread config file.
  */
-static char *print_string(struct context **cnt, char **str, int parm, int threadnr)
+static const char *print_string(struct context **cnt,
+                                char **str ATTRIBUTE_UNUSED, int parm,
+                                int threadnr)
 {
 	int val=config_params[parm].conf_value;
+	const char **cptr0, **cptr1;
 	
 	/* strcmp does not like NULL so we have to check for this also */
-	if (threadnr && *(char **)((void*)cnt[threadnr]+(int)val)!=NULL &&
-	    *(char **)((void*)cnt[0]+(int)val)!=NULL &&	    
-	    !strcmp(*(char **)((void*)cnt[threadnr]+(int)val), *(char **)((void*)cnt[0]+(int)val)))
+	cptr0 = (const char **)((char *)cnt[0] + val);
+	cptr1 = (const char **)((char *)cnt[threadnr] + val);
+	if ((threadnr) && (*cptr0 != NULL) && (*cptr1 != NULL) && (!strcmp(*cptr0, *cptr1)))
 		return NULL;
 
-	return *(char **)((void*)cnt[threadnr]+(int)val);
+	return *cptr1;
 }
 
-static char *print_int(struct context **cnt, char **str, int parm, int threadnr)
+static const char *print_int(struct context **cnt, char **str ATTRIBUTE_UNUSED,
+                             int parm, int threadnr)
 {
 	static char retval[20];
-	int val=config_params[parm].conf_value;
+	int val = config_params[parm].conf_value;
 
 	if (threadnr &&
-	    *(int*)((void*)cnt[threadnr]+(int)val)==
-	    *(int*)((void*)cnt[0]+(int)val))
+	    *(int*)((char *)cnt[threadnr] + val) == *(int*)((char *)cnt[0] + val))
 		return NULL;
 
-	sprintf(retval, "%d", *(int*)((void*)cnt[threadnr]+(int)val));
+	sprintf(retval, "%d", *(int*)((char *)cnt[threadnr] + val));
 
 	return retval;
 }
 
-static char *print_thread(struct context **cnt, char **str, int parm, int threadnr)
+static const char *print_thread(struct context **cnt, char **str,
+                                int parm ATTRIBUTE_UNUSED, int threadnr)
 {
-	char *retval=mymalloc(cnt[0], 1);
+	char *retval;
 	int i=0;
 
-	if (!str || threadnr || !retval)
+	if (!str || threadnr)
 		return NULL;
 
-	retval[0]=0;
+	retval = mymalloc(1);
+	retval[0] = 0;
 	while (cnt[++i]) {
-		retval=myrealloc(cnt[0] ,retval, strlen(retval)+strlen(cnt[i]->conf_filename)+10, "print_thread");
-		sprintf(retval+strlen(retval), "thread %s\n", cnt[i]->conf_filename);
+		retval = myrealloc(retval, strlen(retval) + strlen(cnt[i]->conf_filename)+10, "print_thread");
+		sprintf(retval + strlen(retval), "thread %s\n", cnt[i]->conf_filename);
 	}
-	*str=retval;
+	*str = retval;
 
 	return NULL;
 }
@@ -1672,7 +1777,8 @@ static char *print_thread(struct context **cnt, char **str, int parm, int thread
  * val  - is not used. It is defined to be function header compatible with
  *        copy_int, copy_bool and copy_string.
  */
-static struct context **config_thread(struct context **cnt, char *str, int val)
+static struct context **config_thread(struct context **cnt, const char *str,
+                                      int val ATTRIBUTE_UNUSED)
 {
 	int i;
 	FILE *fp;
@@ -1682,7 +1788,7 @@ static struct context **config_thread(struct context **cnt, char *str, int val)
 
 	fp=fopen(str, "r");
 	if (!fp) {
-		motion_log(cnt[0], LOG_ERR, 1, "Thread config file %s not found",str);
+		motion_log(LOG_ERR, 1, "Thread config file %s not found",str);
 		return cnt;
 	}
 
@@ -1695,10 +1801,10 @@ static struct context **config_thread(struct context **cnt, char *str, int val)
 	 * First thread is 0 so the number of threads is i+1
 	 * plus an extra for the NULL pointer. This gives i+2
 	 */
-	cnt=myrealloc(cnt[0], cnt, sizeof(struct context *)*(i+2), "config_thread");
+	cnt = myrealloc(cnt, sizeof(struct context *)*(i+2), "config_thread");
 
 	/* Now malloc space for an additional context structure for thread nr. i */
-	cnt[i]=mymalloc(cnt[0], sizeof(struct context));
+	cnt[i] = mymalloc(sizeof(struct context));
 	
 	/* And make this an exact clone of the context structure for thread 0 */
 	memcpy(cnt[i], cnt[0], sizeof(struct context));
@@ -1710,18 +1816,13 @@ static struct context **config_thread(struct context **cnt, char *str, int val)
 	 * malloc_strings takes care of this.
 	 */
 	malloc_strings(cnt[i]);
-
-#if 0		/* we will assign threadnr when we launch the threads */
-	/* Set the current thread number */
-	cnt[i]->threadnr=i;
-#endif
 	
 	/* Mark the end if the array of pointers to context structures */
-	cnt[i+1]=NULL;
+	cnt[i+1] = NULL;
 
 	/* process the thread's config file and notify user on console */
 	strcpy(cnt[i]->conf_filename, str);
-	motion_log(cnt[0], LOG_INFO, 0, "Processing config file %s", str);
+	motion_log(LOG_INFO, 0, "Processing config file %s", str);
 	conf_process(cnt+i, fp);
 	
 	/* Finally we close the thread config file */
@@ -1732,14 +1833,14 @@ static struct context **config_thread(struct context **cnt, char *str, int val)
 
 static void usage ()
 {
-	printf("motion Version "VERSION", Copyright 2000 Jeroen Vreeken\n");
+	printf("motion Version "VERSION", Copyright 2000-2005 Jeroen Vreeken/Folkert van Heusden/Kenneth Lavrsen\n");
 	printf("\nusage:\tmotion [options]\n");
 	printf("\n\n");
 	printf("Possible options:\n\n");
 	printf("-n\t\tRun in non-daemon mode.\n");
 	printf("-s\t\tRun in setup mode.\n");
 	printf("-c config\tFull path and filename of config file.\n");
-	printf("-d \t\tDebug mode.\n");
+	printf("-d level\t\tDebug mode.\n");
 	printf("-h\t\tShow this screen.\n");
 	printf("\n");
 	printf("Motion is configured using a config file only. If none is supplied,\n");
