@@ -26,13 +26,13 @@ struct config {
 	int rotate_deg;
 	int max_changes;
 	int threshold_tune;
-	char *output_normal;
+	const char *output_normal;
 	int motion_img;
 	int output_all;
 	int gap;
 	int maxmpegtime;
 	int snapshot_interval;
-	char *locate;
+	const char *locate;
 	int input;
 	int norm;
 	int frame_limit;
@@ -43,7 +43,7 @@ struct config {
 	int mingap;
 	int lightswitch;
 	int nightcomp;
-	int low_cpu;
+	unsigned int low_cpu;
 	int nochild;
 	int autobright;
 	int brightness;
@@ -59,7 +59,8 @@ struct config {
 	int ffmpeg_cap_motion;
 	int ffmpeg_bps;
 	int ffmpeg_vbr;
-	char *ffmpeg_video_codec;
+	int ffmpeg_deinterlace;
+	const char *ffmpeg_video_codec;
 	int webcam_port;
 	int webcam_quality;
 	int webcam_motion;
@@ -69,71 +70,71 @@ struct config {
 	int control_port;
 	int control_localhost;
 	int control_html_output;
-	char *control_authentication;
+	const char *control_authentication;
 	int frequency;
 	int tuner_number;
 	int timelapse;
-	char *timelapse_mode; 
+	const char *timelapse_mode; 
 #ifdef __freebsd__
-	char *tuner_device;
+	const char *tuner_device;
 #endif
-	char *video_device;
-	char *vidpipe;
-	char *filepath;
-	char *jpegpath;
-	char *mpegpath;
-	char *snappath;
-	char *timepath;
+	const char *video_device;
+	const char *vidpipe;
+	const char *filepath;
+	const char *jpegpath;
+	const char *mpegpath;
+	const char *snappath;
+	const char *timepath;
 	char *on_event_start;
 	char *on_event_end;
-	char *mask_file;
+	const char *mask_file;
 	int smart_mask_speed;
 	int sql_log_image;
 	int sql_log_snapshot;
 	int sql_log_mpeg;
 	int sql_log_timelapse;
-	char *mysql_db;
-	char *mysql_host;
-	char *mysql_user;
-	char *mysql_password;
+	const char *sql_query;
+	const char *mysql_db;
+	const char *mysql_host;
+	const char *mysql_user;
+	const char *mysql_password;
 	char *on_picture_save;
 	char *on_motion_detected;
 	char *on_movie_start;
 	char *on_movie_end;
-	char *motionvidpipe;
-	char *netcam_url;
-	char *netcam_userpass;
-	char *netcam_proxy;
-	char *pgsql_db;
-	char *pgsql_host;
-	char *pgsql_user;
-	char *pgsql_password;
+	const char *motionvidpipe;
+	const char *netcam_url;
+	const char *netcam_userpass;
+	const char *netcam_proxy;
+	const char *pgsql_db;
+	const char *pgsql_host;
+	const char *pgsql_user;
+	const char *pgsql_password;
 	int pgsql_port;
 	int text_changes;
-	char *text_left;
-	char *text_right;
+	const char *text_left;
+	const char *text_right;
+	const char *text_event;
 	int text_double;
-	char *despeckle;
+	const char *despeckle;
 	int minimum_motion_frames;
 	// int debug_parameter;
 	int argc;
 	char **argv;
 };
 
-extern struct config conf;
-
 /** 
  * typedef for a param copy function. 
  */
-typedef struct context ** (* conf_copy_func)(struct context **, char *, int);
-typedef char *(* conf_print_func)(struct context **, char **, int, int);
+typedef struct context ** (* conf_copy_func)(struct context **, const char *, int);
+typedef const char *(* conf_print_func)(struct context **, char **, int, int);
 
 /**
  * description for parameters in the config file
  */
 typedef struct {
-	char * param_name;	/* name for this parameter             */
-	char * param_help;	/* short explanation for parameter */
+	const char * param_name;	/* name for this parameter             */
+	const char * param_help;	/* short explanation for parameter */
 	int conf_value;	/* pointer to a field in struct context */
 	conf_copy_func  copy;	/* a function to set the value in 'config' */
 	conf_print_func print;	/* a function to output the value to a file */
@@ -143,12 +144,12 @@ typedef struct {
 extern config_param config_params[];
 
 struct context **conf_load (struct context **);
-struct context **conf_cmdparse(struct context **, char *, char *);
-char *config_type(config_param *);
+struct context **conf_cmdparse(struct context **, const char *, const char *);
+const char *config_type(config_param *);
 void conf_print (struct context **);
 void malloc_strings (struct context *);
-char *mystrdup(char *);
-char *mystrcpy(char *, char *);
-struct context **copy_string(struct context **, char *, int);
+char *mystrdup(const char *);
+char *mystrcpy(char *, const char *);
+struct context **copy_string(struct context **, const char *, int);
 
 #endif /* _INCLUDE_CONF_H */
