@@ -9,17 +9,13 @@
 
 */
 
+#include "ffmpeg.h"           /* must be first to avoid 'shadow' warning */
 #include "motion.h"
 #include "event.h"
-
-#ifdef __freebsd__
-#include "video_freebsd.h"
-#else
+#ifndef __freebsd__
 #include "video.h"
-#endif /* __freebsd__ */
-
+#endif
 #include "picture.h"
-#include "ffmpeg.h"
 
 /*
  *	Various functions (most doing the actual action)
@@ -31,14 +27,10 @@
  * that the fork inherited from the parent in order not to pass
  * the open handles on to the shell
  */
-static void exec_command(struct context *cnt, char *command, char *arg)
+static void exec_command(struct context *cnt, char *command, char *filename, int filetype)
 {
 	char stamp[PATH_MAX];
-	mystrftime(cnt ,stamp, sizeof(stamp), command, cnt->currenttime);
-	if (arg) {
-		strcat(stamp, " ");
-		strcat(stamp, arg);
-	}
+	mystrftime(cnt, stamp, sizeof(stamp), command, cnt->currenttime_tm, filename, filetype);
 	
 	if (!fork()) {
 		int i;
@@ -46,61 +38,79 @@ static void exec_command(struct context *cnt, char *command, char *arg)
 		/* Detach from parent */
 		setsid();
 
-		/* Close any file descripter except console because we will like to see error messages */
-		for (i=getdtablesize(); i>2; --i)
+		/*
+		 * Close any file descriptor except console because we will
+		 * like to see error messages
+		 */
+		for (i = getdtablesize(); i > 2; --i)
 			close(i);
 		
 		execl("/bin/sh", "sh", "-c", stamp, " &", NULL);
 
 		/* if above function succeeds the program never reach here */
-		motion_log(cnt, LOG_ERR, 1, "Unable to start external command '%s' with parameters '%s'",stamp,arg);
+		motion_log(LOG_ERR, 1, "Unable to start external command '%s'", stamp);
 
 		exit(1);
 	}
 	else if (cnt->conf.setup_mode)
-		motion_log(cnt, -1, 0, "Executing external command '%s'", stamp);
+		motion_log(-1, 0, "Executing external command '%s'", stamp);
 }
 
 /* 
  *	Event handlers
  */
 
-static void event_newfile(struct context *cnt, int type, unsigned char *dummy,
-                          char *filename, void *ftype, struct tm *tm)
+static void event_newfile(struct context *cnt ATTRIBUTE_UNUSED,
+            int type ATTRIBUTE_UNUSED, unsigned char *dummy ATTRIBUTE_UNUSED,
+            char *filename, void *ftype, struct tm *tm ATTRIBUTE_UNUSED)
 {
-	motion_log(cnt, -1, 0, "File of type %ld saved to: %s", (unsigned long)ftype, filename);
+	motion_log(-1, 0, "File of type %ld saved to: %s", (unsigned long)ftype, filename);
 }
 
-static void event_motion(struct context *cnt, int type, unsigned char *dummy,
-                         char *filename, void *ftype, struct tm *tm)
+
+static void event_beep(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *dummy ATTRIBUTE_UNUSED,
+            char *filename ATTRIBUTE_UNUSED,
+            void *ftype ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (!cnt->conf.quiet)
 		printf("\a");
 }
 
-static void on_picture_save_command(struct context *cnt, int type, unsigned char *dummy,
-                                    char *filename, void *arg, struct tm *tm)
+/* on_picture_save_command handles both on_picture_save and on_movie_start
+ * If arg = FTYPE_IMAGE_ANY on_picture_save script is executed
+ * If arg = FTYPE_MPEG_ANY on_movie_start script is executed
+ * The scripts are executed with the filename of picture or movie appended
+ * to the config parameter.
+ */
+static void on_picture_save_command(struct context *cnt,
+            int type ATTRIBUTE_UNUSED, unsigned char *dummy ATTRIBUTE_UNUSED,
+            char *filename, void *arg, struct tm *tm ATTRIBUTE_UNUSED)
 {
-	int ftype = (unsigned long)arg;	
+	int filetype = (unsigned long)arg;	
 
-	if ((ftype & FTYPE_IMAGE_ANY) != 0 && cnt->conf.on_picture_save)
-		exec_command(cnt, cnt->conf.on_picture_save, filename);
+	if ((filetype & FTYPE_IMAGE_ANY) != 0 && cnt->conf.on_picture_save)
+		exec_command(cnt, cnt->conf.on_picture_save, filename, filetype);
 
-	if ((ftype & FTYPE_MPEG_ANY) != 0 && cnt->conf.on_movie_start)
-		exec_command(cnt, cnt->conf.on_movie_start, filename);
+	if ((filetype & FTYPE_MPEG_ANY) != 0 && cnt->conf.on_movie_start)
+		exec_command(cnt, cnt->conf.on_movie_start, filename, filetype);
 }
 
-static void on_motion_detected_command(struct context *cnt, int type, unsigned char *dummy1,
-                                       char *dummy2, void *dummy3, struct tm *tm)
+static void on_motion_detected_command(struct context *cnt,
+            int type ATTRIBUTE_UNUSED, unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->conf.on_motion_detected)
-		exec_command(cnt, cnt->conf.on_motion_detected, 0);
+		exec_command(cnt, cnt->conf.on_motion_detected, NULL, 0);
 }
 
 #if defined(HAVE_MYSQL) || defined(HAVE_PGSQL)
 
-static void event_sqlnewfile(struct context *cnt, int type, unsigned char *dummy,
-                             char *filename, void *arg, struct tm *tm)
+static void event_sqlnewfile(struct context *cnt, int type  ATTRIBUTE_UNUSED,
+            unsigned char *dummy ATTRIBUTE_UNUSED,
+            char *filename, void *arg, struct tm *tm ATTRIBUTE_UNUSED)
 {
 	int sqltype = (unsigned long)arg;
 
@@ -112,25 +122,19 @@ static void event_sqlnewfile(struct context *cnt, int type, unsigned char *dummy
 	 * for the sqlquery and timestr when we actually need it.
 	 */
 	{
-		char sqlquery[512];
+		char sqlquery[PATH_MAX];
 		char timestr[20];
+		char eventtimestr[20];
 	
-		strftime(timestr, sizeof(timestr), "%Y-%m-%d %T", cnt->currenttime);
-	
-		sprintf(sqlquery,
-		    "insert into security(camera, filename, frame, file_type, "
-		    "time_stamp %s) values('%d', '%s', '%d', '%d', '%s' %s%s%s)",
-		    cnt->conf.text_left ? ", text_left" : "" ,
-		    cnt->threadnr, filename, cnt->shots, sqltype,  timestr,
-		    cnt->conf.text_left ? ", '" : "" ,
-		    cnt->conf.text_left ? cnt->conf.text_left : "" ,
-		    cnt->conf.text_left ? "'" : ""
-		    );
+		strftime(timestr, sizeof(timestr), "%Y-%m-%d %T", cnt->currenttime_tm);
+		strftime(eventtimestr, sizeof(eventtimestr), "%Y-%m-%d %T", cnt->eventtime_tm);
+		mystrftime(cnt, sqlquery, sizeof(sqlquery), cnt->conf.sql_query, cnt->currenttime_tm, filename, sqltype);
+		
 
 #ifdef HAVE_MYSQL
 		if (cnt->conf.mysql_db) {
 			if (mysql_query(cnt->database, sqlquery) != 0)
-				motion_log(cnt, LOG_ERR, 1, "Mysql query failed");
+				motion_log(LOG_ERR, 1, "Mysql query failed");
 		}
 #endif /* HAVE_MYSQL */
 
@@ -141,7 +145,7 @@ static void event_sqlnewfile(struct context *cnt, int type, unsigned char *dummy
 			res = PQexec(cnt->database_pg, sqlquery);
 
 			if (PQresultStatus(res) != PGRES_COMMAND_OK) {
-				motion_log(cnt, LOG_ERR, 1, "PGSQL query failed");
+				motion_log(LOG_ERR, 1, "PGSQL query failed");
 				PQclear(res);
 			}
 		}
@@ -153,59 +157,67 @@ static void event_sqlnewfile(struct context *cnt, int type, unsigned char *dummy
 #endif /* defined HAVE_MYSQL || defined HAVE_PGSQL */
 
 
-static void event_firstmotion(struct context *cnt, int type, unsigned char *dummy1,
-                              char *dummy2, void *dummy3, struct tm *tm)
+static void on_event_start_command(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->conf.on_event_start)
-		exec_command(cnt, cnt->conf.on_event_start, 0);
+		exec_command(cnt, cnt->conf.on_event_start, NULL, 0);
 }
 
-static void on_event_end_command(struct context *cnt, int type, unsigned char *dummy1,
-                                 char *dummy2, void *dummy3, struct tm *tm)
+static void on_event_end_command(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->conf.on_event_end)
-		exec_command(cnt, cnt->conf.on_event_end, 0);
+		exec_command(cnt, cnt->conf.on_event_end, NULL, 0);
 }
 
-static void event_stop_webcam(struct context *cnt, int type, unsigned char *dummy1,
-                              char *dummy2, void *dummy3, struct tm *tm)
+static void event_stop_webcam(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->conf.webcam_port){
 		webcam_stop(cnt);
 	}
 }
 
-static void event_webcam_put(struct context *cnt, int type, unsigned char *img,
-                             char *dummy1, void *dummy2, struct tm *tm)
+static void event_webcam_put(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *img, char *dummy1 ATTRIBUTE_UNUSED,
+            void *dummy2 ATTRIBUTE_UNUSED, struct tm *tm ATTRIBUTE_UNUSED)
 {
-	if (cnt->conf.webcam_port) {
+	if (cnt->conf.webcam_port)
 		webcam_put(cnt, img);
-	}
 }
 
 #ifndef WITHOUT_V4L
 #ifndef __freebsd__
-static void event_vid_putpipe(struct context *cnt, int type, unsigned char *img,
-                              char *dummy, void *devpipe, struct tm *tm)
+static void event_vid_putpipe(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *img, char *dummy ATTRIBUTE_UNUSED, void *devpipe,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (*(int *)devpipe >= 0) {
 		if (vid_putpipe(*(int *)devpipe, img, cnt->imgs.size) == -1)
-			motion_log(cnt, LOG_ERR, 1, "Failed to put image into video pipe");
+			motion_log(LOG_ERR, 1, "Failed to put image into video pipe");
 	}
 }
 #endif /* __freebsd__ */
 #endif /* WITHOUT_V4L */
 
 
-char *imageext(struct context *cnt)
+const char *imageext(struct context *cnt)
 {
 	if (cnt->conf.ppm)
 		return "ppm";
 	return "jpg";
 }
 
-static void event_image_detect(struct context *cnt, int type, unsigned char *newimg,
-                               char *dummy1, void *dummy2, struct tm *currenttime)
+static void event_image_detect(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *newimg, char *dummy1 ATTRIBUTE_UNUSED,
+            void *dummy2 ATTRIBUTE_UNUSED, struct tm *currenttime_tm)
 {
 	struct config *conf=&cnt->conf;
 	char fullfilename[PATH_MAX];
@@ -214,7 +226,7 @@ static void event_image_detect(struct context *cnt, int type, unsigned char *new
 	char filenamem[PATH_MAX];
 
 	if (conf->motion_img || cnt->new_img==NEWIMG_ON || cnt->preview_shot) {
-		char *jpegpath;
+		const char *jpegpath;
 
 		/* conf.jpegpath would normally be defined but if someone deleted it by control interface
 		   it is better to revert to the default than fail */
@@ -223,7 +235,7 @@ static void event_image_detect(struct context *cnt, int type, unsigned char *new
 		else
 			jpegpath = DEF_JPEGPATH;
 			
-		mystrftime(cnt, filename, sizeof(filename), jpegpath, currenttime);
+		mystrftime(cnt, filename, sizeof(filename), jpegpath, currenttime_tm, NULL, 0);
 		/* motion images gets same name as normal images plus an appended 'm' */
 		sprintf(filenamem, "%sm", filename);
 		sprintf(fullfilename, "%s/%s.%s", cnt->conf.filepath, filename, imageext(cnt));
@@ -237,8 +249,9 @@ static void event_image_detect(struct context *cnt, int type, unsigned char *new
 	}
 }
 
-static void event_image_snapshot(struct context *cnt, int type, unsigned char *img,
-                                 char *dummy1, void *dummy2, struct tm *currenttime)
+static void event_image_snapshot(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *img, char *dummy1 ATTRIBUTE_UNUSED,
+            void *dummy2 ATTRIBUTE_UNUSED, struct tm *currenttime_tm)
 {
 	char fullfilename[PATH_MAX];
 
@@ -246,7 +259,7 @@ static void event_image_snapshot(struct context *cnt, int type, unsigned char *i
 		char filename[PATH_MAX];
 		char filepath[PATH_MAX];
 		char linkpath[PATH_MAX];
-		char *snappath;
+		const char *snappath;
 		/* conf.snappath would normally be defined but if someone deleted it by control interface
 		   it is better to revert to the default than fail */
 		if (cnt->conf.snappath)
@@ -254,7 +267,7 @@ static void event_image_snapshot(struct context *cnt, int type, unsigned char *i
 		else
 			snappath = DEF_SNAPPATH;
 			
-		mystrftime(cnt, filepath, sizeof(filepath), snappath, currenttime);
+		mystrftime(cnt, filepath, sizeof(filepath), snappath, currenttime_tm, NULL, 0);
 		sprintf(filename, "%s.%s", filepath, imageext(cnt));
 		sprintf(fullfilename, "%s/%s", cnt->conf.filepath, filename);
 		put_picture(cnt, fullfilename, img, FTYPE_IMAGE_SNAPSHOT);
@@ -264,7 +277,7 @@ static void event_image_snapshot(struct context *cnt, int type, unsigned char *i
 		sprintf(linkpath, "%s/lastsnap.%s", cnt->conf.filepath, imageext(cnt));
 		remove(linkpath);
 		if (symlink(filename, linkpath)) {
-			motion_log(cnt, LOG_ERR, 1, "Could not create symbolic link [%s]",filename);
+			motion_log(LOG_ERR, 1, "Could not create symbolic link [%s]", filename);
 			return;
 		}
 	} else {
@@ -273,7 +286,7 @@ static void event_image_snapshot(struct context *cnt, int type, unsigned char *i
 		put_picture(cnt, fullfilename, img, FTYPE_IMAGE_SNAPSHOT);
 	}
 
-	cnt->snapshot=0;
+	cnt->snapshot = 0;
 }
 
 #ifdef HAVE_FFMPEG
@@ -283,24 +296,26 @@ static void grey2yuv420p(unsigned char *u, unsigned char *v, int width, int heig
 	memset(v, 128, width*height/4);
 }
 
-static void on_movie_end_command(struct context *cnt, int type, unsigned char *dummy,
-                                 char *filename, void *arg, struct tm *tm)
+static void on_movie_end_command(struct context *cnt, int type ATTRIBUTE_UNUSED,
+                                 unsigned char *dummy ATTRIBUTE_UNUSED, char *filename,
+                                 void *arg, struct tm *tm ATTRIBUTE_UNUSED)
 {
-	int ftype = (unsigned long) arg;
+	int filetype = (unsigned long) arg;
 
-	if ((ftype & FTYPE_MPEG_ANY) && cnt->conf.on_movie_end)
-		exec_command(cnt, cnt->conf.on_movie_end, filename);
+	if ((filetype & FTYPE_MPEG_ANY) && cnt->conf.on_movie_end)
+		exec_command(cnt, cnt->conf.on_movie_end, filename, filetype);
 }
 
-static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *img,
-                                 char *dummy1, void *dummy2, struct tm *currenttime)
+static void event_ffmpeg_newfile(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *img, char *dummy1 ATTRIBUTE_UNUSED,
+            void *dummy2 ATTRIBUTE_UNUSED, struct tm *currenttime_tm)
 {
 	int width=cnt->imgs.width;
 	int height=cnt->imgs.height;
 	unsigned char *convbuf, *y, *u, *v;
 	int fps;
 	char stamp[PATH_MAX];
-	char *mpegpath;
+	const char *mpegpath;
 
 	if (!cnt->conf.ffmpeg_cap_new && !cnt->conf.ffmpeg_cap_motion)
 		return;
@@ -311,14 +326,16 @@ static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *i
 		mpegpath = cnt->conf.mpegpath;
 	else
 		mpegpath = DEF_MPEGPATH;
-	mystrftime(cnt, stamp, sizeof(stamp), mpegpath, currenttime);
+
+	mystrftime(cnt, stamp, sizeof(stamp), mpegpath, currenttime_tm, NULL, 0);
+
 	/* motion mpegs get the same name as normal mpegs plus an appended 'm' */
 	sprintf(cnt->motionfilename, "%s/%sm", cnt->conf.filepath, stamp);
 	sprintf(cnt->newfilename, "%s/%s", cnt->conf.filepath, stamp);
 
 	if (cnt->conf.ffmpeg_cap_new) {
 		if (cnt->imgs.type==VIDEO_PALETTE_GREY) {
-			convbuf=mymalloc(cnt, (width*height)/2);
+			convbuf=mymalloc((width*height)/2);
 			y=img;
 			u=convbuf;
 			v=convbuf+(width*height)/4;
@@ -338,10 +355,10 @@ static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *i
 		if (fps<2)
 			fps=2;
 		if ( (cnt->ffmpeg_new =
-		      ffmpeg_open(cnt, cnt->conf.ffmpeg_video_codec, cnt->newfilename, y, u, v,
+		      ffmpeg_open((char *)cnt->conf.ffmpeg_video_codec, cnt->newfilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, fps, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL) {
-			motion_log(cnt, LOG_ERR, 1, "ffopen_open error creating file [%s]",cnt->newfilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]",cnt->newfilename);
 			cnt->finish=1;
 			return;
 		}
@@ -350,7 +367,7 @@ static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *i
 	}
 	if (cnt->conf.ffmpeg_cap_motion) {
 		if (cnt->imgs.type==VIDEO_PALETTE_GREY) {
-			convbuf=mymalloc(cnt, (width*height)/2);
+			convbuf=mymalloc((width*height)/2);
 			y=cnt->imgs.out;
 			u=convbuf;
 			v=convbuf+(width*height)/4;
@@ -370,10 +387,10 @@ static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *i
 		if (fps<2)
 			fps=2;
 		if ( (cnt->ffmpeg_motion =
-		      ffmpeg_open(cnt ,cnt->conf.ffmpeg_video_codec, cnt->motionfilename, y, u, v,
+		      ffmpeg_open((char *)cnt->conf.ffmpeg_video_codec, cnt->motionfilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, fps, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL){
-			motion_log(cnt, LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->motionfilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->motionfilename);
 			cnt->finish=1;
 			return;
 		}
@@ -382,15 +399,18 @@ static void event_ffmpeg_newfile(struct context *cnt, int type, unsigned char *i
 	}
 }
 
-static void event_ffmpeg_timelapse(struct context *cnt, int type, unsigned char *img,
-                                   char *dummy1, void *dummy2, struct tm *currenttime)
+static void event_ffmpeg_timelapse(struct context *cnt,
+            int type ATTRIBUTE_UNUSED, unsigned char *img,
+            char *dummy1 ATTRIBUTE_UNUSED, void *dummy2 ATTRIBUTE_UNUSED,
+            struct tm *currenttime_tm)
 {
 	int width = cnt->imgs.width;
 	int height = cnt->imgs.height;
 	unsigned char *convbuf, *y, *u, *v;
 
 	if (!cnt->ffmpeg_timelapse) {
-		char tmp[PATH_MAX], *timepath;
+		char tmp[PATH_MAX];
+		const char *timepath;
 
 		/* conf.timepath would normally be defined but if someone deleted it by control interface
 		   it is better to revert to the default than fail */
@@ -399,10 +419,11 @@ static void event_ffmpeg_timelapse(struct context *cnt, int type, unsigned char
 		else
 			timepath = DEF_TIMEPATH;
 		
-		mystrftime(cnt, tmp, sizeof(tmp), timepath, currenttime);
+		mystrftime(cnt, tmp, sizeof(tmp), timepath, currenttime_tm, NULL, 0);
 		sprintf(cnt->timelapsefilename, "%s/%s", cnt->conf.filepath, tmp);
+		
 		if (cnt->imgs.type == VIDEO_PALETTE_GREY) {
-			convbuf = mymalloc(cnt, (width*height)/2);
+			convbuf = mymalloc((width*height)/2);
 			y = img;
 			u = convbuf;
 			v = convbuf+(width*height)/4;
@@ -413,14 +434,16 @@ static void event_ffmpeg_timelapse(struct context *cnt, int type, unsigned char
 			u = img+width*height;
 			v = u+(width*height)/4;
 		}
+		
 		if ( (cnt->ffmpeg_timelapse =
-		      ffmpeg_open(cnt ,TIMELAPSE_CODEC, cnt->timelapsefilename, y, u, v,
+		      ffmpeg_open((char *)TIMELAPSE_CODEC, cnt->timelapsefilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, 24, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL) {
-			motion_log(cnt, LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->timelapsefilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->timelapsefilename);
 			cnt->finish=1;
 			return;
 		}
+		
 		cnt->ffmpeg_timelapse->udata = convbuf;
 		event(cnt, EVENT_FILECREATE, NULL, cnt->timelapsefilename, (void *)FTYPE_MPEG_TIMELAPSE, NULL);
 	}
@@ -433,12 +456,13 @@ static void event_ffmpeg_timelapse(struct context *cnt, int type, unsigned char
 		u = img+width*height;
 	
 	v = u+(width*height)/4;
-	ffmpeg_put_other_image(cnt ,cnt->ffmpeg_timelapse, y, u, v);
+	ffmpeg_put_other_image(cnt->ffmpeg_timelapse, y, u, v);
 	
 }
 
-static void event_ffmpeg_put(struct context *cnt, int type, unsigned char *img,
-                             char *dummy1, void *dummy2, struct tm *tm)
+static void event_ffmpeg_put(struct context *cnt, int type ATTRIBUTE_UNUSED,
+            unsigned char *img, char *dummy1 ATTRIBUTE_UNUSED,
+            void *dummy2 ATTRIBUTE_UNUSED, struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->ffmpeg_new)
 	{
@@ -453,16 +477,18 @@ static void event_ffmpeg_put(struct context *cnt, int type, unsigned char *img,
 			u = y + (width * height);
 		
 		v = u + (width * height) / 4;
-		ffmpeg_put_other_image(cnt, cnt->ffmpeg_new, y, u, v);
+		ffmpeg_put_other_image(cnt->ffmpeg_new, y, u, v);
 	}
 	
 	if (cnt->ffmpeg_motion) {
-		ffmpeg_put_image(cnt, cnt->ffmpeg_motion);
+		ffmpeg_put_image(cnt->ffmpeg_motion);
 	}
 }
 
-static void event_ffmpeg_closefile(struct context *cnt, int type, unsigned char *dummy1,
-                                   char *dummy2, void *dummy3, struct tm *tm)
+static void event_ffmpeg_closefile(struct context *cnt,
+            int type ATTRIBUTE_UNUSED, unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	
 	if (cnt->ffmpeg_new) {
@@ -483,8 +509,10 @@ static void event_ffmpeg_closefile(struct context *cnt, int type, unsigned char
 	}
 }
 
-static void event_ffmpeg_timelapseend(struct context *cnt, int type, unsigned char *dummy1,
-                                      char *dummy2, void *dummy3, struct tm *tm)
+static void event_ffmpeg_timelapseend(struct context *cnt,
+            int type ATTRIBUTE_UNUSED, unsigned char *dummy1 ATTRIBUTE_UNUSED,
+            char *dummy2 ATTRIBUTE_UNUSED, void *dummy3 ATTRIBUTE_UNUSED,
+            struct tm *tm ATTRIBUTE_UNUSED)
 {
 	if (cnt->ffmpeg_timelapse) {
 		if (cnt->ffmpeg_timelapse->udata)
@@ -509,23 +537,24 @@ struct event_handlers {
 };
 
 struct event_handlers event_handlers[] = {
-	{
-	EVENT_FILECREATE,
-	event_newfile
-	},
-	{
-	EVENT_FILECREATE,
-	on_picture_save_command
-	},
 #if defined(HAVE_MYSQL) || defined(HAVE_PGSQL) 
 	{
 	EVENT_FILECREATE,
 	event_sqlnewfile
 	},
 #endif
+	{
+	EVENT_FILECREATE,
+	on_picture_save_command
+	},
+	{
+	EVENT_FILECREATE,
+	event_newfile
+	},
+
 	{
 	EVENT_MOTION,
-	event_motion
+	event_beep
 	},
 	{
 	EVENT_MOTION,
@@ -533,7 +562,7 @@ struct event_handlers event_handlers[] = {
 	},
 	{
 	EVENT_FIRSTMOTION,
-	event_firstmotion
+	on_event_start_command
 	},
 	{
 	EVENT_ENDMOTION,
