@@ -12,8 +12,10 @@
 
 #ifdef HAVE_FFMPEG
 
-#include "motion.h"
 #include "ffmpeg.h"
+#include "motion.h"
+
+
 
 #if LIBAVCODEC_BUILD > 4680
 /* FFmpeg after build 4680 doesn't have support for mpeg1 videos with 
@@ -60,9 +62,9 @@
 #define APPEND_PROTO "appfile"
 
 /* Some forward-declarations. */
-void ffmpeg_put_frame(struct context *, struct ffmpeg *, AVFrame *);
+void ffmpeg_put_frame(struct ffmpeg *, AVFrame *);
 void ffmpeg_cleanups(struct ffmpeg *);
-AVFrame *ffmpeg_prepare_frame(struct context *, struct ffmpeg *, unsigned char *, 
+AVFrame *ffmpeg_prepare_frame(struct ffmpeg *, unsigned char *, 
                               unsigned char *, unsigned char *);
 
 /* This is the trailer used to end mpeg1 videos. */
@@ -72,10 +74,10 @@ static unsigned char mpeg1_trailer[] = {0x00, 0x00, 0x01, 0xb7};
  * an ordinary file. The original file open function truncates an existing
  * file, but this version appends to it instead.
  */
-int file_open_append(URLContext *h, const char *filename, int flags)
+static int file_open_append(URLContext *h, const char *filename, int flags)
 {
 	const char *colon;
-	int access, fd;
+	int access_flags, fd;
 
 	/* Skip past the protocol part of filename. */
 	colon = strchr(filename, ':');
@@ -84,14 +86,14 @@ int file_open_append(URLContext *h, const char *filename, int flags)
 	}
 
 	if (flags & URL_RDWR) {
-		access = O_CREAT | O_APPEND | O_RDWR;
+		access_flags = O_CREAT | O_APPEND | O_RDWR;
 	} else if (flags & URL_WRONLY) {
-		access = O_CREAT | O_APPEND | O_WRONLY;
+		access_flags = O_CREAT | O_APPEND | O_WRONLY;
 	} else {
-		access = O_RDONLY;
+		access_flags = O_RDONLY;
 	}
 
-	fd = open(filename, access, 0666);
+	fd = open(filename, access_flags, 0666);
 	if (fd < 0) {
 		return -ENOENT;
 	}
@@ -144,7 +146,7 @@ void ffmpeg_init()
  * the format is avi; for mpeg1 codec, the format is mpeg. The filename has
  * to be passed, because it gets the appropriate extension appended onto it.
  */
-AVOutputFormat *get_oformat(struct context *cnt, const char *codec, char *filename)
+static AVOutputFormat *get_oformat(const char *codec, char *filename)
 {
 	const char *ext;
 	AVOutputFormat *of = NULL;
@@ -170,7 +172,7 @@ AVOutputFormat *get_oformat(struct context *cnt, const char *codec, char *filena
 		}
 #ifdef FFMPEG_NO_NONSTD_MPEG1
 	} else if (strcmp(codec, "mpeg1") == 0) {
-		motion_log(cnt, LOG_ERR, 0, "*** mpeg1 support for normal videos has been disabled ***");
+		motion_log(LOG_ERR, 0, "*** mpeg1 support for normal videos has been disabled ***");
 		return NULL;
 #endif
 	} else if (strcmp(codec, "mpeg4") == 0) {
@@ -184,12 +186,12 @@ AVOutputFormat *get_oformat(struct context *cnt, const char *codec, char *filena
 			of->video_codec = CODEC_ID_MSMPEG4V2;
 		}
 	} else {
-		motion_log(cnt, LOG_ERR, 0, "ffmpeg_video_codec option value %s is not supported", codec);
+		motion_log(LOG_ERR, 0, "ffmpeg_video_codec option value %s is not supported", codec);
 		return NULL;
 	}
 
 	if (!of) {
-		motion_log(cnt, LOG_ERR, 0, "Could not guess format for %s", codec);
+		motion_log(LOG_ERR, 0, "Could not guess format for %s", codec);
 		return NULL;
 	}
 
@@ -203,10 +205,9 @@ AVOutputFormat *get_oformat(struct context *cnt, const char *codec, char *filena
  * mpeg1 with non-standard framerate, the open will fail. Timelapse is a special
  * case and is tested separately.
  */
-struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec, 
-                           char *filename, unsigned char *y, unsigned char *u, 
-                           unsigned char *v, int width, int height, int rate, 
-                           int bps, int vbr)
+struct ffmpeg *ffmpeg_open(char *ffmpeg_video_codec, char *filename,
+                           unsigned char *y, unsigned char *u, unsigned char *v,
+                           int width, int height, int rate, int bps, int vbr)
 {
 	AVCodecContext *c;
 	AVCodec *codec;
@@ -218,7 +219,7 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 	 * FIXME when motion exits we should close the movie to ensure that
 	 * ffmpeg is freed.
 	 */
-	ffmpeg = mymalloc(cnt, sizeof(struct ffmpeg));
+	ffmpeg = mymalloc(sizeof(struct ffmpeg));
 	memset(ffmpeg, 0, sizeof(struct ffmpeg));
 
 	ffmpeg->vbr = vbr;
@@ -229,13 +230,13 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 	/* allocation the output media context */
 	ffmpeg->oc = av_mallocz(sizeof(AVFormatContext));
 	if (!ffmpeg->oc) {
-		motion_log(cnt, LOG_ERR, 1, "Memory error while allocating output media context");
+		motion_log(LOG_ERR, 1, "Memory error while allocating output media context");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
 
 	/* Setup output format */
-	ffmpeg->oc->oformat = get_oformat(cnt, ffmpeg_video_codec, filename);
+	ffmpeg->oc->oformat = get_oformat(ffmpeg_video_codec, filename);
 	if (!ffmpeg->oc->oformat) {
 		ffmpeg_cleanups(ffmpeg);
 		return NULL;
@@ -248,13 +249,13 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 	if (ffmpeg->oc->oformat->video_codec != CODEC_ID_NONE) {
 		ffmpeg->video_st = av_new_stream(ffmpeg->oc, 0);
 		if (!ffmpeg->video_st) {
-			motion_log(cnt, LOG_ERR, 1, "av_new_stream - could not alloc stream");
+			motion_log(LOG_ERR, 1, "av_new_stream - could not alloc stream");
 			ffmpeg_cleanups(ffmpeg);
 			return (NULL);
 		}
 	} else {
 		/* We did not get a proper video codec. */
-		motion_log(cnt, LOG_ERR, 0, "Failed to obtain a proper video codec");
+		motion_log(LOG_ERR, 0, "Failed to obtain a proper video codec");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
@@ -287,7 +288,7 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 	/* set intra frame distance in frames depending on codec */
 	c->gop_size = is_mpeg1 ? 10 : 12;
 	
-	/* some formats want stream headers to be seperate */
+	/* some formats want stream headers to be separate */
 	if(!strcmp(ffmpeg->oc->oformat->name, "mp4") || 
 	   !strcmp(ffmpeg->oc->oformat->name, "mov") ||
 	   !strcmp(ffmpeg->oc->oformat->name, "3gp")) {
@@ -296,7 +297,7 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 
 	/* set the output parameters (must be done even if no parameters). */
 	if (av_set_parameters(ffmpeg->oc, NULL) < 0) {
-		motion_log(cnt, LOG_ERR, 0, "ffmpeg av_set_parameters error: Invalid output format parameters");
+		motion_log(LOG_ERR, 0, "ffmpeg av_set_parameters error: Invalid output format parameters");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
@@ -308,7 +309,7 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 		codec and allocate the necessary encode buffers */
 	codec = avcodec_find_encoder(c->codec_id);
 	if (!codec) {
-		motion_log(cnt, LOG_ERR, 1, "Codec not found");
+		motion_log(LOG_ERR, 1, "Codec not found");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
@@ -318,7 +319,7 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 
 	/* open the codec */
 	if (avcodec_open(c, codec) < 0) {
-		motion_log(cnt, LOG_ERR, 1, "avcodec_open - could not open codec");
+		motion_log(LOG_ERR, 1, "avcodec_open - could not open codec");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
@@ -328,13 +329,13 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 		/* allocate output buffer */
 		/* XXX: API change will be done */
 		ffmpeg->video_outbuf_size = 200000;
-		ffmpeg->video_outbuf = mymalloc(cnt, ffmpeg->video_outbuf_size);
+		ffmpeg->video_outbuf = mymalloc(ffmpeg->video_outbuf_size);
 	}
 
 	/* allocate the encoded raw picture */
 	ffmpeg->picture = avcodec_alloc_frame();
 	if (!ffmpeg->picture) {
-		motion_log(cnt, LOG_ERR, 1, "avcodec_alloc_frame - could not alloc frame");
+		motion_log(LOG_ERR, 1, "avcodec_alloc_frame - could not alloc frame");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
@@ -369,27 +370,27 @@ struct ffmpeg *ffmpeg_open(struct context *cnt, char *ffmpeg_video_codec,
 			/* path did not exist? */
 			if (errno == ENOENT) {
 				/* create path for file (don't use file_proto)... */
-				if (create_path(cnt, filename) == -1) {
+				if (create_path(filename) == -1) {
 					ffmpeg_cleanups(ffmpeg);
 					return (NULL);
 				}
 
 				/* and retry opening the file (use file_proto) */
 				if (url_fopen(&ffmpeg->oc->pb, file_proto, URL_WRONLY) < 0) {
-					motion_log(cnt, LOG_ERR, 1, "url_fopen - error opening file %s",filename);
+					motion_log(LOG_ERR, 1, "url_fopen - error opening file %s",filename);
 					ffmpeg_cleanups(ffmpeg);
 					return (NULL);
 				}
 				/* Permission denied */
 			} else if (errno ==  EACCES) {
-				motion_log(cnt, LOG_ERR, 1,
+				motion_log(LOG_ERR, 1,
 				           "url_fopen - error opening file %s"
 				           " ... check access rights to target directory", filename);
 				/* create path for file... */
 				ffmpeg_cleanups(ffmpeg);
 				return (NULL);
 			} else {
-				motion_log(cnt, LOG_ERR, 1, "Error opening file %s", filename);
+				motion_log(LOG_ERR, 1, "Error opening file %s", filename);
 				ffmpeg_cleanups(ffmpeg);
 				return (NULL);
 			}
@@ -428,7 +429,9 @@ void ffmpeg_cleanups(struct ffmpeg *ffmpeg)
 */
 	/* free the stream */
 	av_free(ffmpeg->oc);
-
+#if LIBAVFORMAT_BUILD >= 4629
+	free(ffmpeg->c);
+#endif
 	free(ffmpeg);
 }
 
@@ -459,27 +462,28 @@ void ffmpeg_close(struct ffmpeg *ffmpeg)
 
 	/* free the stream */
 	av_free(ffmpeg->oc);
-
+#if LIBAVFORMAT_BUILD >= 4629
+	free(ffmpeg->c);
+#endif
 	free(ffmpeg);
 }
 
 /* Puts the image pointed to by ffmpeg->picture. */
-void ffmpeg_put_image(struct context *cnt, struct ffmpeg *ffmpeg) 
+void ffmpeg_put_image(struct ffmpeg *ffmpeg) 
 {
-	ffmpeg_put_frame(cnt, ffmpeg, ffmpeg->picture);
+	ffmpeg_put_frame(ffmpeg, ffmpeg->picture);
 }
 
 /* Puts an arbitrary picture defined by y, u and v. */
-void ffmpeg_put_other_image(struct context *cnt, struct ffmpeg *ffmpeg, 
-                            unsigned char *y, unsigned char *u, 
-                            unsigned char *v)
+void ffmpeg_put_other_image(struct ffmpeg *ffmpeg, unsigned char *y,
+                            unsigned char *u, unsigned char *v)
 {
 	AVFrame *picture;
 	/* allocate the encoded raw picture */
-	picture = ffmpeg_prepare_frame(cnt, ffmpeg, y, u, v);
+	picture = ffmpeg_prepare_frame(ffmpeg, y, u, v);
 
 	if (picture) {
-		ffmpeg_put_frame(cnt, ffmpeg, picture);
+		ffmpeg_put_frame(ffmpeg, picture);
 		free(picture);
 	}
 }
@@ -487,7 +491,7 @@ void ffmpeg_put_other_image(struct context *cnt, struct ffmpeg *ffmpeg,
 /* Encodes and writes a video frame using the av_write_frame API. This is
  * a helper function for ffmpeg_put_image and ffmpeg_put_other_image. 
  */
-void ffmpeg_put_frame(struct context *cnt, struct ffmpeg *ffmpeg, AVFrame *pic)
+void ffmpeg_put_frame(struct ffmpeg *ffmpeg, AVFrame *pic)
 {
 	int out_size, ret;
 #ifdef FFMPEG_AVWRITEFRAME_NEWAPI
@@ -536,7 +540,7 @@ void ffmpeg_put_frame(struct context *cnt, struct ffmpeg *ffmpeg, AVFrame *pic)
 	}
 	
 	if (ret != 0) {
-		motion_log(cnt, LOG_ERR, 1, "Error while writing video frame");
+		motion_log(LOG_ERR, 1, "Error while writing video frame");
 		return;
 	}
 }
@@ -548,15 +552,14 @@ void ffmpeg_put_frame(struct context *cnt, struct ffmpeg *ffmpeg, AVFrame *pic)
  *
  * The returned AVFrame pointer must be freed after use.
  */
-AVFrame *ffmpeg_prepare_frame(struct context *cnt, struct ffmpeg *ffmpeg, 
-                              unsigned char *y, unsigned char *u, 
-                              unsigned char *v)
+AVFrame *ffmpeg_prepare_frame(struct ffmpeg *ffmpeg, unsigned char *y,
+                              unsigned char *u, unsigned char *v)
 {
 	AVFrame *picture;
 
 	picture = avcodec_alloc_frame();
 	if (!picture) {
-		motion_log(cnt, LOG_ERR, 1, "Could not alloc frame");
+		motion_log(LOG_ERR, 1, "Could not alloc frame");
 		return NULL;
 	}
 
@@ -576,4 +579,43 @@ AVFrame *ffmpeg_prepare_frame(struct context *cnt, struct ffmpeg *ffmpeg,
 	return picture;
 }
 
+
+/** ffmpeg_deinterlace
+ *      Make the image suitable for deinterlacing using ffmpeg, then deinterlace the picture.
+ * 
+ * Parameters
+ *      img     image in YUV420P format
+ *      width   image width in pixels
+ *      height  image height in pixels
+ *
+ * Returns
+ *      Function returns nothing.
+ *      img     contains deinterlaced image
+ */
+void ffmpeg_deinterlace(unsigned char *img, int width, int height)
+{
+	AVFrame *picture;
+	int width2 = width / 2;
+	
+	picture = avcodec_alloc_frame();
+	if (!picture) {
+		motion_log(LOG_ERR, 1, "Could not alloc frame");
+		return;
+	}
+	
+	picture->data[0] = img;
+	picture->data[1] = img+width*height;
+	picture->data[2] = picture->data[1]+(width*height)/4;
+	picture->linesize[0] = width;
+	picture->linesize[1] = width2;
+	picture->linesize[2] = width2;
+	
+	/* We assume using 'PIX_FMT_YUV420P' always */
+	avpicture_deinterlace((AVPicture *)picture, (AVPicture *)picture, PIX_FMT_YUV420P, width, height);
+	
+	av_free(picture);
+	
+	return;
+}
+
 #endif /* HAVE_FFMPEG */
