@@ -27,7 +27,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include "netcam_wget.h"
 
 #define MINVAL(x, y) ((x) < (y) ? (x) : (y))
 
@@ -66,17 +65,17 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
    All trailing whitespace is stripped from the header, and it is
    zero-terminated.  */
    
-int header_get (netcam_context_ptr netcam, char **hdr, enum header_get_flags flags)
+int header_get(netcam_context_ptr netcam, char **hdr, enum header_get_flags flags)
 {
 	int i;
 	int bufsize = 80;
 
-	*hdr = (char *)mymalloc (netcam->cnt, bufsize);
+	*hdr = (char *)mymalloc(bufsize);
 	for (i = 0; 1; i++) {
 		int res;
 		/* #### Use DO_REALLOC?  */
 		if (i > bufsize - 1)
-			*hdr = (char *)myrealloc (netcam->cnt, *hdr, (bufsize <<= 1), "");
+			*hdr = (char *)myrealloc(*hdr, (bufsize <<= 1), "");
 
 		res = RBUF_READCHAR (netcam, *hdr + i);
 
@@ -88,7 +87,7 @@ int header_get (netcam_context_ptr netcam, char **hdr, enum header_get_flags fla
 					/* If the header is non-empty, we need to check if
 					it continues on to the other line.  We do that by
 					peeking at the next character.  */
-					res = rbuf_peek (netcam, &next);
+					res = rbuf_peek(netcam, &next);
 
 					if (res == 0)
 					{
@@ -109,7 +108,7 @@ int header_get (netcam_context_ptr netcam, char **hdr, enum header_get_flags fla
 				/* Strip trailing whitespace.  (*hdr)[i] is the newline;
 				decrement I until it points to the last available
 				whitespace.  */
-				while (i > 0 && isspace ((*hdr)[i - 1]))
+				while (i > 0 && isspace((*hdr)[i - 1]))
 					--i;
 					
 				(*hdr)[i] = '\0';
@@ -134,7 +133,7 @@ int header_get (netcam_context_ptr netcam, char **hdr, enum header_get_flags fla
    the whitespace, and call PROCFUN with the arguments of HEADER's
    contents (after the `:' and space) and ARG.  Otherwise, return 0.  */
 int header_process (const char *header, const char *name,
-		int (*procfun) (const char *, void *),
+		int (*procfun)(const char *, void *),
 		void *arg)
 {
 	/* Check whether HEADER matches NAME.  */
@@ -152,7 +151,7 @@ int header_process (const char *header, const char *name,
 
 /* Extract a long integer from HEADER and store it to CLOSURE.  If an
    error is encountered, return 0, else 1.  */
-int header_extract_number (const char *header, void *closure)
+int header_extract_number(const char *header, void *closure)
 {
 	const char *p = header;
 	long result;
@@ -167,31 +166,27 @@ int header_extract_number (const char *header, void *closure)
 	/* Skip trailing whitespace. */
 	p += skip_lws (p);
 
+	/* We return the value, even if a format error follows */
+	*(long *)closure = result;
+
 	/* Indicate failure if trailing garbage is present. */
 	if (*p)
 		return 0;
 
-	*(long *)closure = result;
 	return 1;
 }
 
 /* Strdup HEADER, and place the pointer to CLOSURE.  */
-int header_strdup (const char *header, void *closure)
+int header_strdup(const char *header, void *closure)
 {
-	*(char **)closure = strdup (header);
+	*(char **)closure = strdup(header);
 	return 1;
 }
 
-/* Write the value 1 into the integer pointed to by CLOSURE.  */
-int header_exists (const char *header, void *closure)
-{
-	*(int *)closure = 1;
-	return 1;
-}
 
 /* Skip LWS (linear white space), if present.  Returns number of
    characters to skip.  */
-int skip_lws (const char *string)
+int skip_lws(const char *string)
 {
 	const char *p = string;
 
@@ -207,7 +202,6 @@ int skip_lws (const char *string)
 	to STORE.  STORE will be 0-terminated, and must point to a writable
 	buffer of at least 1+BASE64_LENGTH(length) bytes.  
 */
-
 void base64_encode(const char *s, char *store, int length)
 {
 	/* Conversion table.  */
@@ -223,7 +217,7 @@ void base64_encode(const char *s, char *store, int length)
 	};
 	
 	int i;
-	unsigned char *p = (unsigned char *) store;
+	unsigned char *p = (unsigned char *)store;
 
 	/* Transform the 3x8 bits to 4x6 bits, as required by base64.  */
 	for (i = 0; i < length; i += 3) {
@@ -244,16 +238,16 @@ void base64_encode(const char *s, char *store, int length)
 	*p = '\0';
 }
 
-char *strdupdelim (const char *beg, const char *end)
+char *strdupdelim(const char *beg, const char *end)
 {
-	char *res = (char *)mymalloc (NULL, end - beg + 1);
+	char *res = (char *)mymalloc(end - beg + 1);
 	memcpy (res, beg, end - beg);
 
 	res[end - beg] = '\0';
 	return res;
 }
 
-int http_process_type (const char *hdr, void *arg)
+int http_process_type(const char *hdr, void *arg)
 {
 	char **result = (char **)arg;
 	/* Locate P on `;' or the terminating zero, whichever comes first. */
@@ -271,20 +265,20 @@ int http_process_type (const char *hdr, void *arg)
 
 /* This is a simple implementation of buffering IO-read functions.  */
 
-void rbuf_initialize (netcam_context_ptr netcam)
+void rbuf_initialize(netcam_context_ptr netcam)
 {
 	netcam->response->buffer_pos = netcam->response->buffer;
 	netcam->response->buffer_left = 0;
 }
 
-int rbuf_read_bufferful (netcam_context_ptr netcam)
+int rbuf_read_bufferful(netcam_context_ptr netcam)
 {
-	return netcam_recv (netcam, netcam->response->buffer,
-	                    sizeof (netcam->response->buffer), 0);
+	return netcam_recv(netcam, netcam->response->buffer,
+	                   sizeof (netcam->response->buffer), 0);
 }
 
 /* Like rbuf_readchar(), only don't move the buffer position.  */
-int rbuf_peek (netcam_context_ptr netcam, char *store)
+int rbuf_peek(netcam_context_ptr netcam, char *store)
 {
 	if (!netcam->response->buffer_left) {
 		int res;
@@ -308,15 +302,15 @@ int rbuf_peek (netcam_context_ptr netcam, char *store)
 	Returns the number of bytes actually copied.  If the buffer is
 	empty, 0 is returned.  
 */
-int rbuf_flush (netcam_context_ptr netcam, char *where, int maxsize)
+int rbuf_flush(netcam_context_ptr netcam, char *where, int maxsize)
 {
 	if (!netcam->response->buffer_left)
 		return 0;
 	else {
-		int howmuch = MINVAL (netcam->response->buffer_left, maxsize);
+		int howmuch = MINVAL ((int)netcam->response->buffer_left, maxsize);
 
 		if (where)
-			memcpy (where, netcam->response->buffer_pos, howmuch);
+			memcpy(where, netcam->response->buffer_pos, howmuch);
 
 		netcam->response->buffer_left -= howmuch;
 		netcam->response->buffer_pos += howmuch;
@@ -332,7 +326,8 @@ void rbuf_discard (netcam_context_ptr netcam)
 }
 
 /* Get the HTTP result code */
-int http_result_code (const char *header) {
+int http_result_code(const char *header)
+{
 	char *cptr;
 
 	/* assure the header starts out right */
