@@ -7,7 +7,7 @@
  *	See also the file 'COPYING'.
  *
  */
-#include "motion.h"
+
 #include "picture.h"
 #include "event.h"
 
@@ -53,7 +53,7 @@ METHODDEF(void) term_destination(j_compress_ptr cinfo)
         dest->jpegsize = dest->bufsize - dest->pub.free_in_buffer;
 }
 
-GLOBAL(void) jpeg_mem_dest(j_compress_ptr cinfo, JOCTET* buf, size_t bufsize)
+static GLOBAL(void) jpeg_mem_dest(j_compress_ptr cinfo, JOCTET* buf, size_t bufsize)
 {
         mem_dest_ptr dest;
 
@@ -74,7 +74,7 @@ GLOBAL(void) jpeg_mem_dest(j_compress_ptr cinfo, JOCTET* buf, size_t bufsize)
         dest->jpegsize = 0;
 }
 
-GLOBAL(int) jpeg_mem_size(j_compress_ptr cinfo)
+static GLOBAL(int) jpeg_mem_size(j_compress_ptr cinfo)
 {
         mem_dest_ptr dest = (mem_dest_ptr) cinfo->dest;
         return dest->jpegsize;
@@ -86,13 +86,13 @@ GLOBAL(int) jpeg_mem_size(j_compress_ptr cinfo)
  * Inputs:
  * - image_size is the size of the input image buffer.
  * - input_image is the image in YUV420P format.
- * - width and height are the dimentions of the image
+ * - width and height are the dimensions of the image
  * - quality is the jpeg encoding quality 0-100%
  * Output:
  * - dest_image is a pointer to the jpeg image buffer
  * Returns buffer size of jpeg image     
  */
-int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
+static int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
                             unsigned char *input_image, int width, int height, int quality)
 {
 	int i, j, jpeg_image_size;
@@ -154,13 +154,13 @@ int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
  * Inputs:
  * - image_size is the size of the input image buffer.
  * - input_image is the image in grayscale format.
- * - width and height are the dimentions of the image
+ * - width and height are the dimensions of the image
  * - quality is the jpeg encoding quality 0-100%
  * Output:
  * - dest_image is a pointer to the jpeg image buffer
  * Returns buffer size of jpeg image     
  */
-int put_jpeg_grey_memory(unsigned char *dest_image, int image_size, unsigned char *input_image, int width, int height, int quality)
+static int put_jpeg_grey_memory(unsigned char *dest_image, int image_size, unsigned char *input_image, int width, int height, int quality)
 {
 	int y, dest_image_size;
 	JSAMPROW row_ptr[1];
@@ -200,13 +200,13 @@ int put_jpeg_grey_memory(unsigned char *dest_image, int image_size, unsigned cha
  * it to an already open file.
  * Inputs:
  * - image is the image in YUV420P format.
- * - width and height are the dimentions of the image
+ * - width and height are the dimensions of the image
  * - quality is the jpeg encoding quality 0-100%
  * Output:
  * - The jpeg is written directly to the file given by the file pointer fp
  * Returns nothing
  */
-void put_jpeg_yuv420p_file(FILE *fp, unsigned char *image, int width, int height, int quality)
+static void put_jpeg_yuv420p_file(FILE *fp, unsigned char *image, int width, int height, int quality)
 {
 	int i,j;
 
@@ -264,13 +264,13 @@ void put_jpeg_yuv420p_file(FILE *fp, unsigned char *image, int width, int height
  * it to an already open file.
  * Inputs:
  * - image is the image in greyscale format.
- * - width and height are the dimentions of the image
+ * - width and height are the dimensions of the image
  * - quality is the jpeg encoding quality 0-100%
  * Output:
  * - The jpeg is written directly to the file given by the file pointer fp
  * Returns nothing
  */
-void put_jpeg_grey_file(FILE *picture, unsigned char *image, int width, int height, int quality)
+static void put_jpeg_grey_file(FILE *picture, unsigned char *image, int width, int height, int quality)
 {
 	int y;
 	JSAMPROW row_ptr[1];
@@ -306,12 +306,12 @@ void put_jpeg_grey_file(FILE *picture, unsigned char *image, int width, int heig
  * it to an already open file.
  * Inputs:
  * - image is the image in YUV420P format.
- * - width and height are the dimentions of the image
+ * - width and height are the dimensions of the image
  * Output:
  * - The PPM is written directly to the file given by the file pointer fp
  * Returns nothing
  */
-void put_ppm_bgr24_file(FILE *picture, unsigned char *image, int width, int height)
+static void put_ppm_bgr24_file(FILE *picture, unsigned char *image, int width, int height)
 {
 	int x, y;
 	unsigned char *l=image;
@@ -479,15 +479,16 @@ void overlay_largest_label(struct context *cnt, unsigned char *out)
  *   converted image it put
  * Function returns the dest_image_size if successful. Otherwise 0.
  */ 
-int put_picture_memory(struct context *cnt, unsigned char* dest_image, int image_size, unsigned char *image, int quality)
+int put_picture_memory(struct context *cnt, unsigned char* dest_image, int image_size,
+                       unsigned char *image, int quality)
 {
 	switch (cnt->imgs.type) {
-       		case VIDEO_PALETTE_YUV420P:
-               		return put_jpeg_yuv420p_memory(dest_image, image_size, image,
-               		                               cnt->imgs.width, cnt->imgs.height, quality);
-       		case VIDEO_PALETTE_GREY:
-               		return put_jpeg_grey_memory(dest_image, image_size, image,
-               		                            cnt->imgs.width, cnt->imgs.height, quality);
+		case VIDEO_PALETTE_YUV420P:
+			return put_jpeg_yuv420p_memory(dest_image, image_size, image,
+			                               cnt->imgs.width, cnt->imgs.height, quality);
+		case VIDEO_PALETTE_GREY:
+			return put_jpeg_grey_memory(dest_image, image_size, image,
+			                            cnt->imgs.width, cnt->imgs.height, quality);
 	}
 
 	return 0;
@@ -514,18 +515,19 @@ void put_picture(struct context *cnt, char *file, unsigned char *image, int ftyp
 {
 	FILE *picture;
 
-	picture=myfopen(cnt ,file, "w");
+	picture = myfopen(file, "w");
 	
 	if (!picture) {
 		/* Report to syslog - suggest solution if the problem is access rights to target dir */
 		if (errno ==  EACCES) {
-			motion_log(cnt, LOG_ERR, 1,
+			motion_log(LOG_ERR, 1,
 			           "Can't write picture to file %s - check access rights to target directory", file);
+			motion_log(LOG_ERR, 1, "Thread is going to finish due to this fatal error");
 			cnt->finish = 1;
 			return;
 		} else {
 			/* If target dir is temporarily unavailable we may survive */
-			motion_log(cnt, LOG_ERR, 1, "Can't write picture to file %s", file);
+			motion_log(LOG_ERR, 1, "Can't write picture to file %s", file);
 			return;
 		}
 	}
@@ -536,62 +538,62 @@ void put_picture(struct context *cnt, char *file, unsigned char *image, int ftyp
 }
 
 /* Get the pgm file used as fixed mask */
-unsigned char *get_pgm(struct context *cnt, FILE *picture, int width, int height)
+unsigned char *get_pgm(FILE *picture, int width, int height)
 {
-	int x=0 ,y=0, maxval;
+	int x = 0 ,y = 0, maxval;
 	char line[256];
 	unsigned char *image;
 
 	line[255]=0;
 	
 	if (!fgets(line, 255, picture)) {
-		motion_log(cnt, LOG_ERR, 1, "Could not read from ppm file");
+		motion_log(LOG_ERR, 1, "Could not read from ppm file");
 		return NULL;
 	}
 	
 	if (strncmp(line, "P5", 2)) {
-		motion_log(cnt, LOG_ERR, 1, "This is not a ppm file, starts with '%s'", line);
+		motion_log(LOG_ERR, 1, "This is not a ppm file, starts with '%s'", line);
 		return NULL;
 	}
 	
 	/* skip comment */
-	line[0]='#';
-	while (line[0]=='#')
+	line[0] = '#';
+	while (line[0] == '#')
 		if (!fgets(line, 255, picture))
 			return NULL;
 
-	
 	/* check size */
 	if (sscanf(line, "%d %d", &x, &y)!=2) {
-		motion_log(cnt, LOG_ERR, 1, "Failed reading size in pgm file");
+		motion_log(LOG_ERR, 1, "Failed reading size in pgm file");
 		return NULL;
 	}
 	
-	if (x!=width || y!=height) {
-		motion_log(cnt, LOG_ERR, 1, "Wrong image size %dx%d% should be %dx%d", x, y, width, height);
+	if (x != width || y != height) {
+		motion_log(LOG_ERR, 1, "Wrong image size %dx%d should be %dx%d", x, y, width, height);
 		return NULL;
 	}
+
 	/* Maximum value */
-	line[0]='#';
-	while (line[0]=='#')
+	line[0] = '#';
+	while (line[0] == '#')
 		if (!fgets(line, 255, picture))
 			return NULL;
 	
 	if (sscanf(line, "%d", &maxval)!=1) {
-		motion_log(cnt, LOG_ERR, 1, "Failed reading maximum value in pgm file");
+		motion_log(LOG_ERR, 1, "Failed reading maximum value in pgm file");
 		return NULL;
 	}
 	
 	/* read data */
 	
-	image=mymalloc(cnt, width*height);
+	image = mymalloc(width * height);
 	
 	for (y=0; y<height; y++) {
-		if (fread(&image[y*width], 1, width, picture)!=width)
-			motion_log(cnt, LOG_ERR, 1, "Failed reading image data from pgm file");
+		if ((int)fread(&image[y * width], 1, width, picture) != width)
+			motion_log(LOG_ERR, 1, "Failed reading image data from pgm file");
 		
 		for (x=0; x<width; x++) {
-			image[y*width+x]=(int)image[y*width+x]*255/maxval;
+			image[y * width + x] = (int)image[y * width + x] * 255 / maxval;
 		}
 	}	
 
@@ -602,19 +604,19 @@ unsigned char *get_pgm(struct context *cnt, FILE *picture, int width, int height
  * creates an empty mask file in the right binary pgm format and
  * and the right size - easy to edit with Gimp or similar tool.
  */
-void put_fixed_mask(struct context *cnt, char *file)
+void put_fixed_mask(struct context *cnt, const char *file)
 {
 	FILE *picture;
 
-	picture=myfopen(cnt, file, "w");
+	picture=myfopen(file, "w");
 	if (!picture) {
 		/* Report to syslog - suggest solution if the problem is access rights to target dir */
 		if (errno ==  EACCES) {
-			motion_log(cnt, LOG_ERR, 1,
+			motion_log(LOG_ERR, 1,
 			           "can't write mask file %s - check access rights to target directory", file);
 		} else {
 			/* If target dir is temporarily unavailable we may survive */
-			motion_log(cnt, LOG_ERR, 1, "can't write mask file %s", file);
+			motion_log(LOG_ERR, 1, "can't write mask file %s", file);
 		}
 		return;
 	}
@@ -626,15 +628,15 @@ void put_fixed_mask(struct context *cnt, char *file)
 	fprintf(picture, "%d\n", 255);
 	
 	/* write pgm image data at once */
-	if (fwrite(cnt->imgs.out, cnt->conf.width, cnt->conf.height, picture) != cnt->conf.height) {
-		motion_log(cnt, LOG_ERR, 1, "Failed writing default mask as pgm file");
+	if ((int)fwrite(cnt->imgs.out, cnt->conf.width, cnt->conf.height, picture) != cnt->conf.height) {
+		motion_log(LOG_ERR, 1, "Failed writing default mask as pgm file");
 		return;
 	}
 	
 	fclose(picture);
 
-	motion_log(cnt, LOG_ERR, 0, "Creating empty mask %s",cnt->conf.mask_file);
-	motion_log(cnt, LOG_ERR, 0, "Please edit this file and re-run motion to enable mask feature");
+	motion_log(LOG_ERR, 0, "Creating empty mask %s",cnt->conf.mask_file);
+	motion_log(LOG_ERR, 0, "Please edit this file and re-run motion to enable mask feature");
 }
 
 /* save 'best' preview_shot */
@@ -643,7 +645,7 @@ void preview_best(struct context *cnt)
 #ifdef HAVE_FFMPEG
 	int use_jpegpath;
 #endif /* HAVE_FFMPEG */
-	char *jpegpath;
+	const char *jpegpath;
 	char previewname[PATH_MAX];
 	char filename[PATH_MAX];
 
@@ -669,11 +671,10 @@ void preview_best(struct context *cnt)
 		if (cnt->conf.jpegpath)
 			jpegpath = cnt->conf.jpegpath;
 		else
-			jpegpath = DEF_JPEGPATH;
+			jpegpath = (char *)DEF_JPEGPATH;
 			
-		mystrftime(cnt, filename, sizeof(filename), jpegpath, cnt->currenttime);
+		mystrftime(cnt, filename, sizeof(filename), jpegpath, cnt->currenttime_tm, NULL, 0);
 		sprintf(previewname, "%s/%s.%s", cnt->conf.filepath, filename, imageext(cnt));
 		put_picture(cnt, previewname, cnt->imgs.preview_buffer , FTYPE_IMAGE);
 	}
-	return;
 }
