@@ -17,7 +17,9 @@ struct trackoptions {
 	int type;
 	char *port;
 	int motorx;
+	int motory;
 	int maxx;
+	int maxy;
 	int stepsize;
 	int speed;
 	int iomojo_id;
@@ -27,12 +29,15 @@ struct trackoptions {
 	int tiltmin;
 	int tiltmax;
 	int minmaxfound;
+	int step_angle_x;
+	int step_angle_y;
+	int move_wait;
 };
 
 extern struct trackoptions track_template;
 
 int track_center(struct context *, int, int, int, int);
-int track_move(struct context *, int, int, int, struct coord *, struct images *, int);
+int track_move(struct context *, int, struct coord *, struct images *, int);
 
 /*
 	Some default values:
@@ -56,6 +61,13 @@ int track_move(struct context *, int, int, int, struct coord *, struct images *,
 #define STEPPER_STATUS_SAFETYL  4
 #define STEPPER_STATUS_SAFETYR  8
 
+#define STEPPER_STATUS_UP       1
+#define STEPPER_STATUS_DOWN     2
+#define STEPPER_STATUS_SAFETYU  4
+#define STEPPER_STATUS_SAFETYD  8
+
+
+
 #define STEPPER_COMMAND_STATUS  0
 #define STEPPER_COMMAND_LEFT_N  1
 #define STEPPER_COMMAND_RIGHT_N 2
@@ -65,6 +77,11 @@ int track_move(struct context *, int, int, int, struct coord *, struct images *,
 #define STEPPER_COMMAND_STOP    6
 #define STEPPER_COMMAND_SPEED   7
 
+#define STEPPER_COMMAND_UP_N    1
+#define STEPPER_COMMAND_DOWN_N  2
+#define STEPPER_COMMAND_UP      3
+#define STEPPER_COMMAND_DOWN    4
+
 
 /*
 	Some defines for the Iomojo Smilecam:
