@@ -10,7 +10,6 @@
 /* Common stuff: */
 #include "motion.h"
 #include "video.h"
-#include "conf.h"
 /* for rotation */
 #include "rotate.h"
 
@@ -18,18 +17,38 @@
 
 /* for the v4l stuff: */
 #include "pwc-ioctl.h"
-#include "sys/mman.h"
-#include "sys/ioctl.h"
-#include "math.h"
+#include <sys/mman.h>
+//#include "sys/ioctl.h"
+#include <math.h>
 #include <sys/utsname.h>
 #include <dirent.h>
 
 #define MAX2(x, y) ((x) > (y) ? (x) : (y))
 #define MIN2(x, y) ((x) < (y) ? (x) : (y))
 
+/* Constants used by auto brightness feature
+ * Defined as constant to make it easier for people to tweak code for a
+ * difficult camera.
+ * The experience gained from people could help improving the feature without
+ * adding too many new options.
+ * AUTOBRIGHT_HYSTERESIS sets the minimum the light intensity must change before
+ * we adjust brigtness.
+ * AUTOBRIGHTS_DAMPER damps the speed with which we adjust the brightness
+ * When the brightness changes a lot we step in large steps and as we approach the
+ * target value we slow down to avoid overshoot and oscillations. If the camera
+ * adjusts too slowly decrease the DAMPER value. If the camera oscillates try
+ * increasing the DAMPER value. DAMPER must be minimum 1.
+ * MAX and MIN are the max and min values of brightness setting we will send to
+ * the camera device.
+ */
+#define AUTOBRIGHT_HYSTERESIS 10
+#define AUTOBRIGHT_DAMPER 5
+#define AUTOBRIGHT_MAX 255
+#define AUTOBRIGHT_MIN 0
+
 static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 {
-	int dev=viddev->fd;
+	int dev = viddev->fd;
 	unsigned char *image = cnt->imgs.image_ring_buffer;
 	struct video_picture vid_pic;
 	int make_change = 0;
@@ -37,7 +56,7 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 	if (cnt->conf.contrast && cnt->conf.contrast != viddev->contrast) {
 
 		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1)
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 
 		make_change = 1;
 		vid_pic.contrast = cnt->conf.contrast * 256;
@@ -48,7 +67,7 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 
 		if (!make_change) {
 			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1)
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 		}
 
 		make_change = 1;
@@ -60,7 +79,7 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 
 		if (!make_change) {
 			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1)
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 		}
 
 		make_change = 1;
@@ -80,8 +99,8 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 		else
 			brightness_target = 128;
 		
-		brightness_window_high = MIN2(brightness_target + 10, 255);
-		brightness_window_low = MAX2(brightness_target - 10, 1);
+		brightness_window_high = MIN2(brightness_target + AUTOBRIGHT_HYSTERESIS, 255);
+		brightness_window_low = MAX2(brightness_target - AUTOBRIGHT_HYSTERESIS, 1);
 		
 		for (i = 0; i < cnt->imgs.motionsize; i += 101) {
 			avg += image[i];
@@ -93,21 +112,23 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 			/* If we already read the VIDIOGPICT - we should not do it again */
 			if (!make_change) {
 				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1)
-					motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+					motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 			}
-			/* average is above window - turn down brigtness - go for the target */
+			/* average is above window - turn down brightness - go for the target */
 			if (avg > brightness_window_high) {
-				step = MIN2((avg - brightness_target)/5+1, viddev->brightness);
-				if (viddev->brightness > step+1) {
+				step = MIN2((avg - brightness_target)/AUTOBRIGHT_DAMPER + 1,
+				             viddev->brightness - AUTOBRIGHT_MIN);
+				if (viddev->brightness > step + 1 - AUTOBRIGHT_MIN) {
 					viddev->brightness -= step;
 					vid_pic.brightness = viddev->brightness * 256;
 					make_change = 1;
 				}
 			}
-			/* average is below window - turn up brigtness - go for the target */
+			/* average is below window - turn up brightness - go for the target */
 			if (avg < brightness_window_low) {
-				step = MIN2((brightness_target - avg)/5+1, 255 - viddev->brightness);
-				if (viddev->brightness < 255-step ) {
+				step = MIN2((brightness_target - avg)/AUTOBRIGHT_DAMPER + 1,
+				             AUTOBRIGHT_MAX - viddev->brightness);
+				if (viddev->brightness < AUTOBRIGHT_MAX - step ) {
 					viddev->brightness += step;
 					vid_pic.brightness = viddev->brightness * 256;
 					make_change = 1;
@@ -118,7 +139,7 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 		if (cnt->conf.brightness && cnt->conf.brightness != viddev->brightness) {
 			if (!make_change) {
 				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1)
-					motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+					motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 			}
 	
 			make_change = 1;
@@ -129,12 +150,12 @@ static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
 
 	if (make_change) {
 		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1)
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSPICT)");
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCSPICT)");
 	}
 }
 		
 
-void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
+static void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
 {
 	unsigned char *src, *dest, *src2, *dest2;
 	int i, j;
@@ -167,7 +188,7 @@ void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int hei
 	}
 }
 
-void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int height)
+static void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int height)
 {
 	unsigned char *y, *u, *v;
 	unsigned char *r, *g, *b;
@@ -213,41 +234,45 @@ void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int h
 */
 
 
-static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, int width, int height, int input, int norm,
-                        unsigned long freq, int tuner_number)
+static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, int width, int height,
+                                int input, int norm, unsigned long freq, int tuner_number)
 {
-	int dev=viddev->fd;
+	int dev = viddev->fd;
 	struct video_capability vid_caps;
 	struct video_channel vid_chnl;
 	struct video_tuner vid_tuner;
-	struct video_window vid_win;
 	struct video_mbuf vid_buf;
 	struct video_mmap vid_mmap;
-	unsigned char *map;
+	void *map;
 
 	if (ioctl (dev, VIDIOCGCAP, &vid_caps) == -1) {
-		motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGCAP)");
+		motion_log(LOG_ERR, 1, "ioctl (VIDIOCGCAP)");
 		return (NULL);
 	}
-	if (vid_caps.type & VID_TYPE_MONOCHROME) viddev->v4l_fmt=VIDEO_PALETTE_GREY;
+
+	if (vid_caps.type & VID_TYPE_MONOCHROME)
+		viddev->v4l_fmt=VIDEO_PALETTE_GREY;
+
 	if (input != IN_DEFAULT) {
 		memset(&vid_chnl, 0, sizeof(struct video_channel));
 		vid_chnl.channel = input;
+
 		if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGCHAN)");
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCGCHAN)");
 		} else {
 			vid_chnl.channel = input;
 			vid_chnl.norm    = norm;
 			if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSCHAN)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCSCHAN)");
 				return (NULL);
 			}
 		}
 	}
+
 	if (freq) {
 		vid_tuner.tuner = tuner_number;
 		if (ioctl (dev, VIDIOCGTUNER, &vid_tuner)==-1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGTUNER)");
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCGTUNER)");
 		} else {
 			if (vid_tuner.flags & VIDEO_TUNER_LOW) {
 				freq=freq*16; /* steps of 1/16 KHz */
@@ -255,48 +280,32 @@ static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, i
 				freq=(freq*10)/625;
 			}
 			if (ioctl(dev, VIDIOCSFREQ, &freq)==-1) {
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
 				return (NULL);
 			}
 			if (cnt->conf.setup_mode)
-				motion_log(cnt, -1, 0, "Frequency set");
+				motion_log(-1, 0, "Frequency set");
 		}
 	}
+
 	if (ioctl (dev, VIDIOCGMBUF, &vid_buf) == -1) {
-		motion_log(cnt, LOG_WARNING, 0, "No mmap falling back on read");
-		if (ioctl (dev, VIDIOCGWIN, &vid_win)== -1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGWIN)");
-			return (NULL);
-		}
-		vid_win.width=width;
-		vid_win.height=height;
-		vid_win.clipcount=0;
-		if (ioctl (dev, VIDIOCSWIN, &vid_win)== -1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSWIN)");
-			return (NULL);
-		}
-		motion_log(cnt, LOG_WARNING, 0, "V4L capturing using read is deprecated!");
-		motion_log(cnt, LOG_WARNING, 0, "Motion now only supports mmap.");
-		motion_log(cnt, LOG_WARNING, 0, "Motion Exits.");
-		exit(1);  //FIXME - This seems like an old oops that was never cleaned out or fixed
-		viddev->v4l_read_img=1;
-		map=mymalloc(cnt ,width*height*3);
-		viddev->v4l_maxbuffer=1;
-		viddev->v4l_curbuffer=0;
-		viddev->v4l_buffers[0]=map;
-	}else{
+		motion_log(LOG_ERR, 0, "ioctl(VIDIOCGMBUF) - Error device does not support memory map");
+		motion_log(LOG_ERR, 0, "V4L capturing using read is deprecated!");
+		motion_log(LOG_ERR, 0, "Motion only supports mmap.");
+		return NULL;
+	} else {
 		map=mmap(0, vid_buf.size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, 0);
 		viddev->size_map=vid_buf.size;
 		if (vid_buf.frames>1) {
 			viddev->v4l_maxbuffer=2;
 			viddev->v4l_buffers[0]=map;
-			viddev->v4l_buffers[1]=map+vid_buf.offsets[1];
+			viddev->v4l_buffers[1]=(unsigned char *)map + vid_buf.offsets[1];
 		} else {
 			viddev->v4l_buffers[0]=map;
 			viddev->v4l_maxbuffer=1;
 		}
 
-		if ((unsigned char *)-1 == (unsigned char *)map) {
+		if (MAP_FAILED == map) {
 			return (NULL);
 		}
 		viddev->v4l_curbuffer=0;
@@ -305,22 +314,22 @@ static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, i
 		vid_mmap.width=width;
 		vid_mmap.height=height;
 		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-			motion_log(cnt, LOG_DEBUG, 1, "Failed with YUV420P, trying YUV422 palette");
+			motion_log(LOG_DEBUG, 1, "Failed with YUV420P, trying YUV422 palette");
 			viddev->v4l_fmt=VIDEO_PALETTE_YUV422;
 			vid_mmap.format=viddev->v4l_fmt;
 			/* Try again... */
 			if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-				motion_log(cnt, LOG_DEBUG, 1, "Failed with YUV422, trying RGB24 palette");
+				motion_log(LOG_DEBUG, 1, "Failed with YUV422, trying RGB24 palette");
 				viddev->v4l_fmt=VIDEO_PALETTE_RGB24;
 				vid_mmap.format=viddev->v4l_fmt;
 				/* Try again... */
 				if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-					motion_log(cnt, LOG_DEBUG, 1, "Failed with RGB24, trying GREYSCALE palette");
+					motion_log(LOG_DEBUG, 1, "Failed with RGB24, trying GREYSCALE palette");
 					viddev->v4l_fmt=VIDEO_PALETTE_GREY;
 					vid_mmap.format=viddev->v4l_fmt;
 					/* Try one last time... */
 					if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-						motion_log(cnt, LOG_ERR, 1, "Failed with all supported palettes "
+						motion_log(LOG_ERR, 1, "Failed with all supported palettes "
 						                            "- giving up");
 						return (NULL);
 					}
@@ -348,81 +357,84 @@ static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, i
 
 
 /**
- * v4l_next fetches a video frame from a v4l device
+ * v4l_next
+ *                v4l_next fetches a video frame from a v4l device
+ *
  * Parameters:
  *     cnt        Pointer to the context for this thread
- *     viddev     Pointer to struct containing video device handle
+ *     viddev     Pointer to struct containing video device handle amd device parameters
  *     map        Pointer to the buffer in which the function puts the new image
  *     width      Width of image in pixels
  *     height     Height of image in pixels
  *
  * Returns
- *     0          Success
- *    -1          Fatal error
- *     1          Non fatal error (not implemented)
+ *     0               Success
+ *    V4L_FATAL_ERROR  Fatal error
+ *    Positive with bit 0 set and bit 1 unset
+ *                     Non fatal error (not implemented)
  */
-int v4l_next(struct context *cnt, struct video_dev *viddev, unsigned char *map, int width, int height)
+static int v4l_next(struct video_dev *viddev, unsigned char *map, int width, int height)
 {
-	int dev=viddev->fd;
-	int frame=viddev->v4l_curbuffer;
+	int dev = viddev->fd;
+	int frame = viddev->v4l_curbuffer;
 	struct video_mmap vid_mmap;
-	unsigned char *cap_map=NULL;
+	unsigned char *cap_map;
 
 	sigset_t  set, old;
 
-	if (viddev->v4l_read_img) {
-		if (read(dev, map, viddev->v4l_bufsize) != viddev->v4l_bufsize)
-			return -1;
-	} else {
-		vid_mmap.format=viddev->v4l_fmt;
-		vid_mmap.width=width;
-		vid_mmap.height=height;
-		/* Block signals during IOCTL */
-		sigemptyset (&set);
-		sigaddset (&set, SIGCHLD);
-		sigaddset (&set, SIGALRM);
-		sigaddset (&set, SIGUSR1);
-		sigaddset (&set, SIGTERM);
-		sigaddset (&set, SIGHUP);
-		pthread_sigmask (SIG_BLOCK, &set, &old);
+	/* MMAP method is used */
+	vid_mmap.format = viddev->v4l_fmt;
+	vid_mmap.width = width;
+	vid_mmap.height = height;
 
-		cap_map=viddev->v4l_buffers[viddev->v4l_curbuffer];
-		viddev->v4l_curbuffer++;
-		if (viddev->v4l_curbuffer >= viddev->v4l_maxbuffer)
-			viddev->v4l_curbuffer=0;
-		vid_mmap.frame=viddev->v4l_curbuffer;
+	/* Block signals during IOCTL */
+	sigemptyset(&set);
+	sigaddset(&set, SIGCHLD);
+	sigaddset(&set, SIGALRM);
+	sigaddset(&set, SIGUSR1);
+	sigaddset(&set, SIGTERM);
+	sigaddset(&set, SIGHUP);
+	pthread_sigmask (SIG_BLOCK, &set, &old);
 
-		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-			motion_log(cnt, LOG_ERR, 1, "mcapture error in proc %d", getpid());
-			sigprocmask (SIG_UNBLOCK, &old, NULL);
-			return (-1);
-		}
+	cap_map = viddev->v4l_buffers[viddev->v4l_curbuffer];
+	viddev->v4l_curbuffer++;
 
-		vid_mmap.frame=frame;
-		if (ioctl(dev, VIDIOCSYNC, &vid_mmap.frame) == -1) {
-			motion_log(cnt, LOG_ERR, 1, "sync error in proc %d", getpid());
-			sigprocmask (SIG_UNBLOCK, &old, NULL);
-		}
-		
-		pthread_sigmask (SIG_UNBLOCK, &old, NULL);        /*undo the signal blocking*/
+	if (viddev->v4l_curbuffer >= viddev->v4l_maxbuffer)
+		viddev->v4l_curbuffer = 0;
+
+	vid_mmap.frame = viddev->v4l_curbuffer;
+
+	if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
+		motion_log(LOG_ERR, 1, "mcapture error in proc %d", getpid());
+		sigprocmask (SIG_UNBLOCK, &old, NULL);
+		return V4L_FATAL_ERROR;
 	}
-	if (!viddev->v4l_read_img) {
-		switch (viddev->v4l_fmt) {
-			case VIDEO_PALETTE_RGB24:
-				rgb24toyuv420p(map, cap_map, width, height);
-				break;
-			case VIDEO_PALETTE_YUV422:
-				yuv422to420p(map, cap_map, width, height);
-				break;
-			default:
-				memcpy(map, cap_map, viddev->v4l_bufsize);
-		}
+
+	vid_mmap.frame=frame;
+
+	if (ioctl(dev, VIDIOCSYNC, &vid_mmap.frame) == -1) {
+		motion_log(LOG_ERR, 1, "sync error in proc %d", getpid());
+		sigprocmask (SIG_UNBLOCK, &old, NULL);
 	}
 
+	pthread_sigmask (SIG_UNBLOCK, &old, NULL);        /*undo the signal blocking*/
+
+	switch (viddev->v4l_fmt) {
+		case VIDEO_PALETTE_RGB24:
+			rgb24toyuv420p(map, cap_map, width, height);
+			break;
+		case VIDEO_PALETTE_YUV422:
+			yuv422to420p(map, cap_map, width, height);
+			break;
+		default:
+			memcpy(map, cap_map, viddev->v4l_bufsize);
+	}
+
+
 	return 0;
 }
 
-void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char *map, int width, int height, int input,
+static void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char *map, int width, int height, int input,
                     int norm, int skip, unsigned long freq, int tuner_number)
 {
 	int dev=viddev->fd;
@@ -436,7 +448,7 @@ void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char
 		if (freq) {
 			vid_tuner.tuner = tuner_number;
 			if (ioctl (dev, VIDIOCGTUNER, &vid_tuner)==-1) {
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGTUNER)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCGTUNER)");
 			} else {
 				if (vid_tuner.flags & VIDEO_TUNER_LOW) {
 					frequnits=freq*16; /* steps of 1/16 KHz */
@@ -444,7 +456,7 @@ void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char
 					frequnits=(freq*10)/625;
 				}
 				if (ioctl(dev, VIDIOCSFREQ, &frequnits)==-1) {
-					motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
+					motion_log(LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
 					return;
 				}
 			}
@@ -453,12 +465,12 @@ void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char
 		vid_chnl.channel = input;
 		
 		if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGCHAN)");
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCGCHAN)");
 		} else {
 			vid_chnl.channel = input;
 			vid_chnl.norm = norm;
 			if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
-				motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSCHAN)");
+				motion_log(LOG_ERR, 1, "ioctl (VIDIOCSCHAN)");
 				return;
 			}
 		}
@@ -470,16 +482,16 @@ void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char
 		viddev->tuner_number=tuner_number;
 		/* skip a few frames if needed */
 		for (i=0; i<skip; i++)
-			v4l_next(cnt, viddev, map, width, height);
+			v4l_next(viddev, map, width, height);
 	} else {
 		/* No round robin - we only adjust picture controls */
 		v4l_picture_controls(cnt, viddev);
 	}
 }
 
-static int v4l_open_vidpipe(struct context *cnt)
+static int v4l_open_vidpipe(void)
 {
-	int pipe = -1;
+	int pipe_fd = -1;
 	char pipepath[255];
 	char buffer[255];
 	char *major;
@@ -487,13 +499,13 @@ static int v4l_open_vidpipe(struct context *cnt)
 	struct utsname uts;
 
 	if (uname(&uts) < 0) {
-		motion_log(cnt, LOG_ERR, 1, "Unable to execute uname");
+		motion_log(LOG_ERR, 1, "Unable to execute uname");
 		return -1;
 	}
 	major = strtok(uts.release, ".");
 	minor = strtok(NULL, ".");
 	if ((major == NULL) || (minor == NULL) || (strcmp(major, "2"))) {
-		motion_log(cnt, LOG_ERR, 1, "Unable to decipher OS version");
+		motion_log(LOG_ERR, 1, "Unable to decipher OS version");
 		return -1;
 	}
 	if (strcmp(minor, "5") < 0) {
@@ -506,22 +518,22 @@ static int v4l_open_vidpipe(struct context *cnt)
 
 		vloopbacks=fopen("/proc/video/vloopback/vloopbacks", "r");
 		if (!vloopbacks) {
-			motion_log(cnt, LOG_ERR, 1, "Failed to open '/proc/video/vloopback/vloopbacks'");
+			motion_log(LOG_ERR, 1, "Failed to open '/proc/video/vloopback/vloopbacks'");
 			return -1;
 		}
 		
 		/* Read vloopback version*/
 		if (!fgets(buffer, 255, vloopbacks)) {
-			motion_log(cnt, LOG_ERR, 1, "Unable to read vloopback version");
+			motion_log(LOG_ERR, 1, "Unable to read vloopback version");
 			return -1;
 		}
 		
 		fprintf(stderr,"\t%s", buffer);
 		
-		/* Read explaination line */
+		/* Read explanation line */
 		
 		if (!fgets(buffer, 255, vloopbacks)) {
-			motion_log(cnt, LOG_ERR, 1, "Unable to read vloopback explanation line");
+			motion_log(LOG_ERR, 1, "Unable to read vloopback explanation line");
 			return -1;
 		}
 		
@@ -535,10 +547,10 @@ static int v4l_open_vidpipe(struct context *cnt)
 				ostatus=strtok(NULL, "\t");
 				if (istatus[0]=='-') {
 					sprintf(pipepath, "/dev/%s", input);
-					pipe=open(pipepath, O_RDWR);
-					if (pipe>=0) {
-						motion_log(cnt, -1, 0, "\tInput:  /dev/%s", input);
-						motion_log(cnt, -1, 0, "\tOutput: /dev/%s", output);
+					pipe_fd=open(pipepath, O_RDWR);
+					if (pipe_fd>=0) {
+						motion_log(-1, 0, "\tInput:  /dev/%s", input);
+						motion_log(-1, 0, "\tOutput: /dev/%s", output);
 						break;
 					}
 				}
@@ -556,7 +568,7 @@ static int v4l_open_vidpipe(struct context *cnt)
 		int tnum;
 
 		if ((dir=opendir(prefix))== NULL) {
-			motion_log(cnt, LOG_ERR, 1, "Failed to open '%s'", prefix);
+			motion_log(LOG_ERR, 1, "Failed to open '%s'", prefix);
 			return -1;
 		}
 		while ((dirp=readdir(dir)) != NULL) {
@@ -591,10 +603,10 @@ static int v4l_open_vidpipe(struct context *cnt)
 						strcat(buffer, dirp->d_name);
 						if ((tfd=open(buffer, O_RDWR))>=0) {
 							strcpy(pipepath, buffer);
-							if (pipe>=0) {
-								close(pipe);
+							if (pipe_fd>=0) {
+								close(pipe_fd);
 							}
-							pipe = tfd;
+							pipe_fd = tfd;
 							low = tnum;
 						}
 					}
@@ -603,20 +615,20 @@ static int v4l_open_vidpipe(struct context *cnt)
 			}
 		}
 		closedir(dir);
-		if (pipe >= 0)
-			motion_log(cnt, -1, 0, "Opened input of %s", pipepath);
+		if (pipe_fd >= 0)
+			motion_log(-1, 0, "Opened input of %s", pipepath);
 	}
-	return pipe;
+	return pipe_fd;
 }
 
-static int v4l_startpipe(struct context *cnt, char *devname, int width, int height, int type)
+static int v4l_startpipe(char *devname, int width, int height, int type)
 {
 	int dev;
 	struct video_picture vid_pic;
 	struct video_window vid_win;
 
 	if (!strcmp(devname, "-")) {
-		dev=v4l_open_vidpipe(cnt);
+		dev=v4l_open_vidpipe();
 	} else {
 		dev=open(devname, O_RDWR);
 	}
@@ -624,22 +636,22 @@ static int v4l_startpipe(struct context *cnt, char *devname, int width, int heig
 		return(-1);
 
 	if (ioctl(dev, VIDIOCGPICT, &vid_pic)== -1) {
-		motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
+		motion_log(LOG_ERR, 1, "ioctl (VIDIOCGPICT)");
 		return(-1);
 	}
 	vid_pic.palette=type;
 	if (ioctl(dev, VIDIOCSPICT, &vid_pic)== -1) {
-		motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSPICT)");
+		motion_log(LOG_ERR, 1, "ioctl (VIDIOCSPICT)");
 		return(-1);
 	}
 	if (ioctl(dev, VIDIOCGWIN, &vid_win)== -1) {
-		motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCGWIN)");
+		motion_log(LOG_ERR, 1, "ioctl (VIDIOCGWIN)");
 		return(-1);
 	}
 	vid_win.height=height;
 	vid_win.width=width;
 	if (ioctl(dev, VIDIOCSWIN, &vid_win)== -1) {
-		motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSWIN)");
+		motion_log(LOG_ERR, 1, "ioctl (VIDIOCSWIN)");
 		return(-1);
 	}
 	return dev;
@@ -649,30 +661,46 @@ static int v4l_putpipe (int dev, unsigned char *image, int size)
 {
 	return write(dev, image, size);
 }
+#endif /*WITHOUT_V4L*/
 
 /*****************************************************************************
 	Wrappers calling the actual capture routines
  *****************************************************************************/
 
-#endif /*WITHOUT_V4L*/
-/* big lock for vid_start */
+/* big lock for vid_start to ensure exclusive access to viddevs while adding 
+ * devices during initialization of each thread
+ */
 pthread_mutex_t vid_mutex;
+
 #ifndef WITHOUT_V4L
 
-/* structure used for per device locking */
+/* Here we setup the viddevs structure which is used globally in the vid_*
+ * functions.
+ */
 struct video_dev **viddevs=NULL;
 
-void vid_init(struct context *cnt)
+/**
+ * vid_init
+ *
+ * Called from motion.c at the very beginning before setting up the threads.
+ * Function prepares the viddevs struct for the threads and the vid_mutex
+ */
+void vid_init(void)
 {
 	if (!viddevs) {
-		viddevs=mymalloc(cnt, sizeof(struct video_dev *));
-		viddevs[0]=NULL;
+		viddevs = mymalloc(sizeof(struct video_dev *));
+		viddevs[0] = NULL;
 	}
 
 	pthread_mutex_init(&vid_mutex, NULL);
 }
 
-/* Called by childs to get rid of open video devices */
+/**
+ * vid_close
+ *
+ * vid_close is called from motion.c when Motion is stopped or restarted
+ * It gets rid of all open video devices. It is called BEFORE vid_cleanup.
+ */
 void vid_close(void)
 {
 	int i=-1;
@@ -684,6 +712,12 @@ void vid_close(void)
 	}
 }
 
+/**
+ * vid_cleanup
+ *
+ * vid_cleanup is called from motion.c when Motion is stopped or restarted
+ * It free all the memory held by the viddevs structs.
+ */
 void vid_cleanup(void)
 {
 	int i=-1;
@@ -698,25 +732,68 @@ void vid_cleanup(void)
 }
 #endif /*WITHOUT_V4L*/
 
-int vid_start (struct context *cnt)
+/**
+ * vid_start
+ *
+ * vid_start setup the capture device. This will be either a V4L device or a netcam.
+ * The function does the following:
+ * - If the camera is a netcam - netcam_start is called and function returns
+ * - Width and height are checked for valid value (multiple of 16)
+ * - Copy the config height and width to the imgs struct. Note that height and width are
+ *   only copied to the from the conf struct to the imgs struct during program startup
+ *   The width and height can no later be changed via http remote control as this would
+ *   require major re-memory allocations of all image buffers.
+ * - Setup basic V4L properties incl palette incl setting 
+ * - Open the device
+ * - Returns the device number.
+ *
+ * Parameters:
+ *     cnt        Pointer to the context for this thread
+ *
+ * "Global" variable
+ *     viddevs    The viddevs struct is "global" within the context of video.c
+ *                and used in functions vid_*.
+ *     vid_mutex  Mutex needed to handle exclusive access to the viddevs struct when
+ *                each thread adds a new video device during startup calling vid_start
+ *
+ * Returns
+ *     device number
+ *     -1 if failed to open device.
+ */
+int vid_start(struct context *cnt)
 {
-	struct config *conf=&cnt->conf;
-	int dev=-1;
+	struct config *conf = &cnt->conf;
+	int dev = -1;
 
 	if (conf->netcam_url) {
 		return netcam_start(cnt);
 	}
 
 #ifndef WITHOUT_V4L
+	/* Start a new block so we can make declarations without breaking good old
+	 * gcc 2.95 or older.
+	 */
 	{
-		int i=-1;
+		int i = -1;
 		int width, height, input, norm, tuner_number;
 		unsigned long frequency;
 
 		/* We use width and height from conf in this function. They will be assigned
 		 * to width and height in imgs here, and cap_width and cap_height in 
 		 * rotate_data won't be set until in rotate_init.
+		 * Motion requires that width and height is a multiple of 16 so we check
+		 * for this first.
 		 */
+		if (conf->width % 16) {
+			motion_log(LOG_ERR, 0, "config image width (%d) is not modulo 16", conf->width);
+			return -1;
+		}
+
+		if (conf->height % 16) {
+			motion_log(LOG_ERR, 0, "config image height (%d) is not modulo 16", conf->height);
+			return -1;
+		}
+
 		width = conf->width;
 		height = conf->height;
 		input = conf->input;
@@ -728,26 +805,31 @@ int vid_start (struct context *cnt)
 
 		/* Transfer width and height from conf to imgs. The imgs values are the ones
 		 * that is used internally in Motion. That way, setting width and height via
-		 * xmlrpc won't screw things up.
+		 * http remote control won't screw things up.
 		 */
 		cnt->imgs.width=width;
 		cnt->imgs.height=height;
 
+		/* First we walk through the already discovered video devices to see
+		 * if we have already setup the same device before. If this is the case
+		 * the device is a Round Robin device and we set the basic settings
+		 * and return the file descriptor
+		 */
 		while (viddevs[++i]) {
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
-			  	int fd;
-				cnt->imgs.type=viddevs[i]->v4l_fmt;
+				int fd;
+				cnt->imgs.type = viddevs[i]->v4l_fmt;
 				switch (cnt->imgs.type) {
 					case VIDEO_PALETTE_GREY:
-						cnt->imgs.motionsize=width*height;
-						cnt->imgs.size=width*height;
+						cnt->imgs.motionsize = width * height;
+						cnt->imgs.size = width * height;
 						break;
 					case VIDEO_PALETTE_RGB24:
 					case VIDEO_PALETTE_YUV422:
-						cnt->imgs.type=VIDEO_PALETTE_YUV420P;
+						cnt->imgs.type = VIDEO_PALETTE_YUV420P;
 					case VIDEO_PALETTE_YUV420P:
-						cnt->imgs.motionsize=width*height;
-						cnt->imgs.size=(width*height*3)/2;
+						cnt->imgs.motionsize = width * height;
+						cnt->imgs.size = (width*height * 3) / 2;
 						break;
 				}
 				fd=viddevs[i]->fd;
@@ -756,91 +838,107 @@ int vid_start (struct context *cnt)
 			}
 		}
 
-		viddevs=myrealloc(cnt, viddevs, sizeof(struct video_dev *)*(i+2), "vid_start");
-		viddevs[i]=mymalloc(cnt, sizeof(struct video_dev));
+		viddevs = myrealloc(viddevs, sizeof(struct video_dev *)*(i+2), "vid_start");
+		viddevs[i] = mymalloc(sizeof(struct video_dev));
 		memset(viddevs[i], 0, sizeof(struct video_dev));
 		viddevs[i+1]=NULL;
 
 		pthread_mutexattr_init(&viddevs[i]->attr);
 		pthread_mutex_init(&viddevs[i]->mutex, NULL);
 
-		dev=open(conf->video_device, O_RDWR);
+		dev = open(conf->video_device, O_RDWR);
+
 		if (dev <0) {
-			motion_log(cnt, LOG_ERR, 1, "Failed to open video device %s", conf->video_device);
-			motion_log(cnt, LOG_ERR, 0, "Motion Exits");
-			exit(1);
+			motion_log(LOG_ERR, 1, "Failed to open video device %s", conf->video_device);
+			return -1;
 		}
 
-		viddevs[i]->video_device=conf->video_device;
-		viddevs[i]->fd=dev;
-		viddevs[i]->input=input;
-		viddevs[i]->height=height;
-		viddevs[i]->width=width;
-		viddevs[i]->freq=frequency;
-		viddevs[i]->tuner_number=tuner_number;
+		viddevs[i]->video_device = conf->video_device;
+		viddevs[i]->fd = dev;
+		viddevs[i]->input = input;
+		viddevs[i]->height = height;
+		viddevs[i]->width = width;
+		viddevs[i]->freq = frequency;
+		viddevs[i]->tuner_number = tuner_number;
 		
 		/* We set brightness, contrast, saturation and hue = 0 so that they only get
 		 * set if the config is not zero.
 		 */
-		viddevs[i]->brightness=0;
-		viddevs[i]->contrast=0;
-		viddevs[i]->saturation=0;
-		viddevs[i]->hue=0;
-		viddevs[i]->owner=-1;
+		viddevs[i]->brightness = 0;
+		viddevs[i]->contrast = 0;
+		viddevs[i]->saturation = 0;
+		viddevs[i]->hue = 0;
+		viddevs[i]->owner = -1;
+		viddevs[i]->v4l_fmt = VIDEO_PALETTE_YUV420P;
 
-		viddevs[i]->v4l_fmt=VIDEO_PALETTE_YUV420P;
-
-		if (!v4l_start (cnt, viddevs[i], width, height, input, norm, frequency, tuner_number)) {
+		if (!v4l_start(cnt, viddevs[i], width, height, input, norm, frequency, tuner_number)) {
 			pthread_mutex_unlock(&vid_mutex);
 			return -1;
 		}
-		cnt->imgs.type=viddevs[i]->v4l_fmt;
+
+		cnt->imgs.type = viddevs[i]->v4l_fmt;
+
 		switch (cnt->imgs.type) {
 			case VIDEO_PALETTE_GREY:
-				cnt->imgs.size=width*height;
-				cnt->imgs.motionsize=width*height;
+				cnt->imgs.size = width*height;
+				cnt->imgs.motionsize = width*height;
 			break;
 			case VIDEO_PALETTE_RGB24:
 			case VIDEO_PALETTE_YUV422:
-				cnt->imgs.type=VIDEO_PALETTE_YUV420P;
+				cnt->imgs.type = VIDEO_PALETTE_YUV420P;
 			case VIDEO_PALETTE_YUV420P:
-				cnt->imgs.size=(width*height*3)/2;
-				cnt->imgs.motionsize=width*height;
+				cnt->imgs.size = (width*height*3)/2;
+				cnt->imgs.motionsize = width*height;
 			break;
 		}
 
 		pthread_mutex_unlock(&vid_mutex);
 	}
 #endif /*WITHOUT_V4L*/	
+
 	return dev;
 }
 
 
 /**
+ * vid_next
+ *
  * vid_next fetches a video frame from a either v4l device or netcam
+ *
  * Parameters:
  *     cnt        Pointer to the context for this thread
  *     map        Pointer to the buffer in which the function puts the new image
  *
+ * Global variable
+ *     viddevs    The viddevs struct is "global" within the context of video.c
+ *                and used in functions vid_*.
  * Returns
- *     0          Success
- *    -1          Fatal V4L error
- *    -2          Fatal Netcam error
- *     1          Non fatal V4L error (not implemented)
- *     2          Non fatal Netcam error
+ *     0                        Success
+ *    -1                        Fatal V4L error
+ *    -2                        Fatal Netcam error
+ *    Positive numbers...
+ *    with bit 0 set            Non fatal V4L error (not implemented)
+ *    with bit 1 set            Non fatal Netcam error
  */
 int vid_next(struct context *cnt, unsigned char *map)
 {
 	struct config *conf=&cnt->conf;
-	int ret = -1;
+	int ret = -1;		
 
 	if (conf->netcam_url) {
+		if (cnt->video_dev == -1)
+			return NETCAM_GENERAL_ERROR;
+
 		ret = netcam_next(cnt, map);
 		return ret;
 	}
 #ifndef WITHOUT_V4L
+
+	/* We start a new block so we can make declarations without breaking
+	 * gcc 2.95 or older
+	 */
 	{
-		int i=-1;
+		int i = -1;
 		int width, height;
 		int dev = cnt->video_dev;
 
@@ -849,25 +947,26 @@ int vid_next(struct context *cnt, unsigned char *map)
 		height = cnt->rotate_data.cap_height;
 		
 		while (viddevs[++i])
-			if (viddevs[i]->fd==dev)
+			if (viddevs[i]->fd == dev)
 				break;
 
 		if (!viddevs[i])
-			return -1;
+			return V4L_FATAL_ERROR;
 
-		if (viddevs[i]->owner!=cnt->threadnr) {
+		if (viddevs[i]->owner != cnt->threadnr) {
 			pthread_mutex_lock(&viddevs[i]->mutex);
-			viddevs[i]->owner=cnt->threadnr;
-			viddevs[i]->frames=conf->roundrobin_frames;
-			cnt->switched=1;
+			viddevs[i]->owner = cnt->threadnr;
+			viddevs[i]->frames = conf->roundrobin_frames;
+			cnt->switched = 1;
 		}
 
 		v4l_set_input(cnt, viddevs[i], map, width, height, conf->input, conf->norm,
 		               conf->roundrobin_skip, conf->frequency, conf->tuner_number);
-		ret = v4l_next(cnt, viddevs[i], map, width, height);
+
+		ret = v4l_next(viddevs[i], map, width, height);
 
 		if (--viddevs[i]->frames <= 0) {
-			viddevs[i]->owner=-1;
+			viddevs[i]->owner = -1;
 			pthread_mutex_unlock(&viddevs[i]->mutex);
 		}
 	
@@ -881,13 +980,13 @@ int vid_next(struct context *cnt, unsigned char *map)
 }
 
 #ifndef WITHOUT_V4L
-int vid_startpipe (struct context *cnt, char *devname, int width, int height, int type)
+int vid_startpipe(const char *devname, int width, int height, int type)
 {
-	return v4l_startpipe(cnt, devname, width, height, type);
+	return v4l_startpipe( (char *)devname, width, height, type);
 }
 
 int vid_putpipe (int dev, unsigned char *image, int size)
 {
-	return v4l_putpipe (dev, image, size);
+	return v4l_putpipe(dev, image, size);
 }
 #endif /*WITHOUT_V4L*/
