@@ -10,15 +10,14 @@
 /* Common stuff: */
 #include "motion.h"
 #include "video_freebsd.h"
-#include "conf.h"
 /* for rotation */
 #include "rotate.h"
 
 #ifndef WITHOUT_V4L
 
 /* for the v4l stuff: */
-#include "sys/mman.h"
-#include "sys/ioctl.h"
+#include <sys/mman.h>
+#include <sys/types.h>
 
 /* Hack from xawtv 4.x */
 
@@ -44,9 +43,6 @@
 #define VIDEO_MPEG          19  /* MPEG1/2 */
 #define VIDEO_FMT_COUNT     20
 
-#define FBTTV_DEF_AUTOBRIGHT   0
-#define FBTTV_DEF_CHANNELSET   2 /* cableirc */
-#define FBTTV_DEF_CHANNEL      3 /* local PBS */
 #define MAX(x,y) ( (x) > (y) ? (x) : (y) )
 #define MIN(x,y) ( (x) < (y) ? (x) : (y) )
 #define array_elem(x) (sizeof(x) / sizeof( (x)[0] ))
@@ -73,22 +69,9 @@ static const struct camparam_st {
 #define CONTR  1
 #define CHROMA 2
 
-#define NTSC_MAX_X        640
-#define NTSC_MAX_Y        480
-#define PAL_MAX_X         768
-#define PAL_MAX_Y         576
-#define PAL                 1
-#define NTSC                2
- 
-#define FBTTV_MAX_WIDTH     MAX(NTSC_MAX_X, PAL_MAX_X)
-#define FBTTV_MAX_HEIGHT    MAX(NTSC_MAX_Y, PAL_MAX_Y)
-#define FBTTV_MIN_WIDTH     2
-#define FBTTV_MIN_HEIGHT    2
-
-
 /* Not tested yet */
 
-void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
+static void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
 {
 	unsigned char *src, *dest, *src2, *dest2;
 	int i, j;
@@ -124,7 +107,7 @@ void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int hei
 
 /* FIXME seems no work with METEOR_GEO_RGB24 , check BPP as well ? */
 
-void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int height)
+static void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int height)
 {
 	unsigned char *y, *u, *v;
 	unsigned char *r, *g, *b;
@@ -157,7 +140,8 @@ void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int h
 			v++;
 		}
 
-		if ((loop & 1) == 1) {
+		if ((loop & 1) == 0) 
+		{
 			u-=width/2;
 			v-=width/2;
 		}
@@ -169,7 +153,9 @@ void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int h
 /*********************************************************************************************
                 CAPTURE CARD STUFF
 **********************************************************************************************/
-/* NOT TESTED YET FIXME
+/* NOT TESTED YET FIXME */
+
+/*
 static int camparam_normalize( int param, int cfg_value, int *ioctl_val ) 
 {
 	int val;
@@ -182,53 +168,131 @@ static int camparam_normalize( int param, int cfg_value, int *ioctl_val )
 	*ioctl_val = val;
 	return cfg_value;
 }
+*/
 
-
-static int set_chroma( struct video_dev *viddev, int new_chroma ) 
+static int set_hue( int viddev, int new_hue )
 {
-	int ioctlval;
+	signed char ioctlval=new_hue;
 
-	new_chroma = camparam_normalize( CHROMA, new_chroma, &ioctlval );
+	if( ioctl( viddev, METEORSHUE, &ioctlval ) < 0 ) {
+                motion_log(LOG_ERR, 1, "METEORSHUE Error setting hue [%d]",new_hue);
+                return -1;
+        }
 
-	if( ioctl( viddev->fd_tuner, BT848_SCSAT, &ioctlval ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "Error setting CHROMA");
+	motion_log(-1, 0, "set hue to [%d]",ioctlval);
+
+	return ioctlval;
+}
+
+static int get_hue( int viddev , int *hue)
+{
+	signed char ioctlval;
+
+	if( ioctl( viddev, METEORGHUE, &ioctlval ) < 0 ) {
+		motion_log(LOG_ERR, 1, "METEORGHUE Error getting hue");
 		return -1;
 	}
 
-	viddev->chroma = new_chroma;
-
-	return 0;
+	*hue = ioctlval; 
+	return ioctlval;
 }
 
-static int set_contrast( struct video_dev *viddev, int new_contrast ) 
+static int set_saturation( int viddev, int new_saturation ) 
 {
-	int ioctlval;
+	unsigned char ioctlval= new_saturation;
 
-	new_contrast = camparam_normalize( CONTR, new_contrast, &ioctlval );
-
-	if( ioctl( viddev->fd_tuner, BT848_SCONT, &ioctlval ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "Error setting contrast");
+	if( ioctl( viddev, METEORSCSAT, &ioctlval ) < 0 ) {
+		motion_log(LOG_ERR, 1, "METEORSCSAT Error setting saturation [%d]",new_saturation);
 		return -1;
 	}
 
-	viddev->contrast = new_contrast;
+	motion_log(-1, 0, "set saturation to [%d]",ioctlval);
 
-	return 0;
+	return ioctlval;
 }
 
+static int get_saturation( int viddev , int *saturation)
+{
+	unsigned char ioctlval;
+
+	if( ioctl( viddev, METEORGCSAT, &ioctlval ) < 0 ) {
+
+		motion_log(LOG_ERR, 1, "METEORGCSAT Error getting saturation");
+		return -1;
+	}
+
+	*saturation = ioctlval;
+	return ioctlval;
+}
+
+static int set_contrast( int viddev, int new_contrast ) 
+{
+	unsigned char ioctlval = new_contrast;
+
+	if( ioctl( viddev, METEORSCONT, &ioctlval ) < 0 ) {
+		motion_log(LOG_ERR, 1, "METEORSCONT Error setting contrast [%d]", new_contrast);
+		return 0;
+	}
+
+	motion_log(-1, 0, "set contrast to [%d]",ioctlval);
+
+	return ioctlval;
+}
+
+static int get_contrast( int viddev, int *contrast )
+{
+	unsigned char ioctlval;
+
+	if( ioctl (viddev, METEORGCONT, &ioctlval ) < 0 ) {
+		motion_log(LOG_ERR, 1, "METEORGCONT Error getting contrast");
+		return -1;
+	}
+
+	*contrast = ioctlval; 
+	return ioctlval;
+}
+
+
+static int set_brightness( int viddev, int new_bright )
+{
+	unsigned char ioctlval = new_bright;
+
+	if( ioctl( viddev, METEORSBRIG, &ioctlval ) < 0 ) {
+		motion_log(LOG_ERR, 1, "METEORSBRIG  brightness [%d]",new_bright);
+		return -1;
+	}
+
+	motion_log(-1, 0, "set brightness to [%d]",ioctlval);
+	
+	return ioctlval;
+}
+
+
+static int get_brightness( int viddev, int *brightness )
+{
+	unsigned char ioctlval;
+
+	if( ioctl( viddev, METEORGBRIG, &ioctlval ) < 0 ) {
+                motion_log(LOG_ERR, 1, "METEORGBRIG  getting brightness");
+                return -1;
+        }
+
+	*brightness = ioctlval;
+	return ioctlval;
+}
 
 // Set channel needed ? FIXME 
-
+/*
 static int set_channel( struct video_dev *viddev, int new_channel ) 
 {
 	int ioctlval;
 
 	ioctlval = new_channel;
 	if( ioctl( viddev->fd_tuner, TVTUNER_SETCHNL, &ioctlval ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "Error channel %d",ioctlval);
+		motion_log(LOG_ERR, 1, "Error channel %d",ioctlval);
 		return -1;
 	} else {
-		motion_log(cnt, LOG_DEBUG, 0, "channel set to %d",ioctlval);
+		motion_log(LOG_DEBUG, 0, "channel set to %d",ioctlval);
 	}
 
 	viddev->channel = new_channel;
@@ -239,7 +303,7 @@ static int set_channel( struct video_dev *viddev, int new_channel )
 
 /* set frequency to tuner */
 
-static int set_freq(struct context *cnt, struct video_dev *viddev, unsigned long freq)
+static int set_freq(struct video_dev *viddev, unsigned long freq)
 {
 	int tuner_fd = viddev->fd_tuner;
 	int old_audio;
@@ -247,12 +311,12 @@ static int set_freq(struct context *cnt, struct video_dev *viddev, unsigned long
 	/* HACK maybe not need it , but seems that is needed to mute before changing frequency */
 
 	if ( ioctl( tuner_fd, BT848_GAUDIO, &old_audio ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "BT848_GAUDIO");
+		motion_log(LOG_ERR, 1, "BT848_GAUDIO");
 		return -1;
 	}
 	
 	if (ioctl(tuner_fd, TVTUNER_SETFREQ, &freq) < 0){
-		motion_log(cnt, LOG_ERR, 1, "Tuning (TVTUNER_SETFREQ) failed , freq [%lu]",freq);
+		motion_log(LOG_ERR, 1, "Tuning (TVTUNER_SETFREQ) failed , freq [%lu]",freq);
 		return -1;
 	}
 
@@ -260,7 +324,7 @@ static int set_freq(struct context *cnt, struct video_dev *viddev, unsigned long
 	if ( old_audio ){
 		old_audio = AUDIO_MUTE;
 		if ( ioctl(tuner_fd , BT848_SAUDIO, &old_audio ) < 0 ) {
-			motion_log(cnt, LOG_ERR, 1, "BT848_SAUDIO %i",old_audio);
+			motion_log(LOG_ERR, 1, "BT848_SAUDIO %i",old_audio);
 			return -1;
 		}
 	}
@@ -277,7 +341,7 @@ static int set_freq(struct context *cnt, struct video_dev *viddev, unsigned long
 	VBI ?! (METEOR_INPUT_DEV_SVIDEO)
 */
 
-static int set_input(struct context *cnt, struct video_dev *viddev, int input)
+static int set_input(struct video_dev *viddev, unsigned short input)
 {
 	int actport;
 	int portdata[] = { METEOR_INPUT_DEV0, METEOR_INPUT_DEV1,
@@ -285,23 +349,23 @@ static int set_input(struct context *cnt, struct video_dev *viddev, int input)
 	                 METEOR_INPUT_DEV_SVIDEO  };
 
 	if( input >= array_elem( portdata ) ) {
-		motion_log(cnt, LOG_WARNING, 0, "Channel Port %d out of range (0-4)",input);
+		motion_log(LOG_WARNING, 0, "Channel Port %d out of range (0-4)",input);
 		input = IN_DEFAULT;
 	}
 
 	actport = portdata[ input ];
 	if( ioctl( viddev->fd_bktr, METEORSINPUT, &actport ) < 0 ) {
 		if( input != IN_DEFAULT ) {
-			motion_log(cnt, LOG_WARNING, 0,
+			motion_log(LOG_WARNING, 0,
 			           "METEORSINPUT %d invalid - Trying default %d ", input, IN_DEFAULT );
 			input = IN_DEFAULT;
 			actport = portdata[ input ];
 			if( ioctl( viddev->fd_bktr, METEORSINPUT, &actport ) < 0 ) {
-				motion_log(cnt, LOG_ERR, 1, "METEORSINPUT %d init", input);
+				motion_log(LOG_ERR, 1, "METEORSINPUT %d init", input);
 				return -1;
 			}
 		} else {
-			motion_log(cnt, LOG_ERR, 1, "METEORSINPUT  %d init",input);
+			motion_log(LOG_ERR, 1, "METEORSINPUT  %d init",input);
 			return -1;
 		}
 	}
@@ -309,18 +373,25 @@ static int set_input(struct context *cnt, struct video_dev *viddev, int input)
 	return 0;
 }
 
-static int set_geometry(struct context *cnt, struct video_dev *viddev, int width , int height , int format)
+static int set_geometry(struct video_dev *viddev, int width, int height)
 {
 	struct meteor_geomet geom;
 
 	geom.columns = width;
 	geom.rows = height;
+
+//	geom.rows = geom.rows / 2;
+//	geom.oformat |= METEOR_GEO_ODD_ONLY;
+
+
 	geom.oformat = METEOR_GEO_YUV_422 | METEOR_GEO_YUV_12;
-	geom.oformat |= METEOR_GEO_EVEN_ONLY;	
+//	geom.oformat |= METEOR_GEO_EVEN_ONLY;	
+	
+//	geom.oformat = 0;
 	geom.frames  = 1;
 
 	if( ioctl( viddev->fd_bktr, METEORSETGEO, &geom ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "Couldn't set the geometry");
+		motion_log(LOG_ERR, 1, "Couldn't set the geometry");
 		return -1;
 	}
 
@@ -328,46 +399,36 @@ static int set_geometry(struct context *cnt, struct video_dev *viddev, int width
 }
 
 /*
-   set input format ( NTSC , PAL , SECAM , etc ... ) not all supported yet FIXME
+   set input format ( PAL, NTSC, SECAM, etc ... )
 */
 
-static int set_input_format(struct context *cnt, struct video_dev *viddev, int newformat) 
+static int set_input_format(struct video_dev *viddev, unsigned short newformat) 
 {
+	int input_format[] = { NORM_PAL_NEW, NORM_NTSC_NEW, NORM_SECAM_NEW, NORM_DEFAULT_NEW};
 	int format;
-	int input_format[] = { NORM_PAL, NORM_NTSC, NORM_SECAM };
  
 	if( newformat >= array_elem( input_format ) ) {
-		motion_log(cnt, LOG_WARNING, 0, "Input format %d out of range (0-2)",newformat );
-		format = NORM_DEFAULT;newformat = 0;
+		motion_log(LOG_WARNING, 0, "Input format %d out of range (0-2)",newformat );
+		format = NORM_DEFAULT_NEW;
+		newformat = 3;
 	} else
 		format = input_format[newformat]; 
 
-	/* FIXME should be BT848SFMT for new BKTR INTERFACES */
-
-	if( ioctl( viddev->fd_bktr, METEORSFMT, &format ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, "METEORSFMT Couldn't set the input format");
-		return -1;
+	if( ioctl( viddev->fd_bktr, BT848SFMT, &format ) < 0 ) {
+		motion_log(LOG_ERR, 1, "BT848SFMT, Couldn't set the input format , try again with default");
+		format = NORM_DEFAULT_NEW;
+		newformat = 3;
+		
+		if( ioctl( viddev->fd_bktr, BT848SFMT, &format ) < 0 ) {
+			motion_log(LOG_ERR, 1, "BT848SFMT, Couldn't set the input format either default");
+			return -1;
+		}
 	}
+	
 	return 0;
 }
 
-/* NOT TESTED YET FIXME
-static int set_brightness( struct video_dev *viddev, int new_bright ) 
-{
-	int ioctlval;
-
-	new_bright = camparam_normalize( BRIGHT, new_bright, &ioctlval );
-	if( ioctl( viddev->fd_tuner, BT848_SBRIG, &ioctlval ) < 0 ) {
-		motion_log(cnt, LOG_ERR, 1, " BT848_SBRIG brightness %d",ioctlval);
-		return -1;
-	}
-	viddev->brightness = new_bright;
-
-	return 0;
-}
-
-
-
+/*
 statict int setup_pixelformat( int bktr )
 {
 	int i;
@@ -379,7 +440,7 @@ statict int setup_pixelformat( int bktr )
 		if( ioctl( bktr, METEORGSUPPIXFMT, &p ) < 0 ){
 			if( errno == EINVAL )
 				break;
-			motion_log(cnt, LOG_ERR, 1, "METEORGSUPPIXFMT getting pixformat %d",i);	
+			motion_log(LOG_ERR, 1, "METEORGSUPPIXFMT getting pixformat %d",i);	
 			return -1;
 		}
 
@@ -388,40 +449,40 @@ statict int setup_pixelformat( int bktr )
 
 	switch ( p.type ){
 		case METEOR_PIXTYPE_RGB:
-			motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB");
+			motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB");
 			switch(p.masks[0]) {
 				case 31744: // 15 bpp 
 					format = p.swap_bytes ? VIDEO_RGB15_LE : VIDEO_RGB15_BE;
-					motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB15");
+					motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB15");
 				break;
 				case 63488: // 16 bpp 
 					format = p.swap_bytes ? VIDEO_RGB16_LE : VIDEO_RGB16_BE;
-					motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB16");
+					motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB16");
 				break;
 				case 16711680: // 24/32 bpp 
 					if (p.Bpp == 3 && p.swap_bytes == 1) {
 						format = VIDEO_BGR24;
-						motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_BGR24");
+						motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_BGR24");
 					} else if (p.Bpp == 4 && p.swap_bytes == 1 && p.swap_shorts == 1) {
 						format = VIDEO_BGR32;
-						motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_BGR32");
+						motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_BGR32");
 						} else if (p.Bpp == 4 && p.swap_bytes == 0 && p.swap_shorts == 0) {
 							format = VIDEO_RGB32;
-							motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB32");
+							motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_RGB VIDEO_RGB32");
 						}
 					}
 				break;
 				case METEOR_PIXTYPE_YUV:
 					format = VIDEO_YUV422P;
-					motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV");
+					motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV");
 				break;
 				case METEOR_PIXTYPE_YUV_12:
 					format = VIDEO_YUV422P;
-					motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV_12");
+					motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV_12");
 					break;
 				case METEOR_PIXTYPE_YUV_PACKED:
 					format = VIDEO_YUV422P;
-					motion_log(cnt, -1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV_PACKED");
+					motion_log(-1, 0, "setup_pixelformat METEOR_PIXTYPE_YUV_PACKED");
 				break;
 	
 			}
@@ -429,20 +490,20 @@ statict int setup_pixelformat( int bktr )
 			if( p.type == METEOR_PIXTYPE_RGB && p.Bpp == 3 ){
 				// Found a good pixeltype -- set it up 
 				if( ioctl( bktr, METEORSACTPIXFMT, &i ) < 0 ){
-					motion_log(cnt, LOG_WARNING, 1, "METEORSACTPIXFMT etting pixformat METEOR_PIXTYPE_RGB Bpp == 3");
+					motion_log(LOG_WARNING, 1, "METEORSACTPIXFMT etting pixformat METEOR_PIXTYPE_RGB Bpp == 3");
 				// Not immediately fatal 
 				}
-				motion_log(cnt, LOG_DEBUG, 0, "input format METEOR_PIXTYPE_RGB %i",i);
+				motion_log(LOG_DEBUG, 0, "input format METEOR_PIXTYPE_RGB %i",i);
 				format=i;
 			}
 
 			if( p.type == METEOR_PIXTYPE_YUV_PACKED ){
 				// Found a good pixeltype -- set it up
 				if( ioctl( bktr, METEORSACTPIXFMT, &i ) < 0 ){
-					motion_log(cnt, LOG_WARNING, 1, "METEORSACTPIXFMT setting pixformat METEOR_PIXTYPE_YUV_PACKED");
+					motion_log(LOG_WARNING, 1, "METEORSACTPIXFMT setting pixformat METEOR_PIXTYPE_YUV_PACKED");
 				// Not immediately fatal
 				} 
-				motion_log(cnt, LOG_DEBUG, 0, "input format METEOR_PIXTYPE_YUV_PACKED %i",i);
+				motion_log(LOG_DEBUG, 0, "input format METEOR_PIXTYPE_YUV_PACKED %i",i);
 				format=i;
 			}
 
@@ -453,6 +514,34 @@ statict int setup_pixelformat( int bktr )
 
 */
 
+static void v4l_picture_controls(struct context *cnt, struct video_dev *viddev)
+{
+	int dev=viddev->fd_bktr;
+
+	if ( (cnt->conf.contrast) && (cnt->conf.contrast != viddev->contrast) ){ 
+		set_contrast(dev,cnt->conf.contrast);
+		viddev->contrast = cnt->conf.contrast;	
+	}
+
+	if ( (cnt->conf.hue) && (cnt->conf.hue != viddev->hue) ) {
+		set_hue(dev, cnt->conf.hue);
+		viddev->hue = cnt->conf.hue;	
+	}
+
+	if ( (cnt->conf.brightness) && 
+	     (cnt->conf.brightness != viddev->brightness)) {
+		set_brightness(dev, cnt->conf.brightness);
+		viddev->brightness = cnt->conf.brightness; 
+	}
+
+	if ( (cnt->conf.saturation ) && 
+	     (cnt->conf.saturation != viddev->saturation) ){
+		set_saturation(dev, cnt->conf.saturation);
+		viddev->saturation = cnt->conf.saturation;
+	}
+}
+
+
 /*******************************************************************************************
 	Video capture routines
 
@@ -468,54 +557,63 @@ statict int setup_pixelformat( int bktr )
  - set_capture_mode
 
 */
-static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width, int height, int input, int norm, unsigned long freq)
+static unsigned char *v4l_start(struct context *cnt, struct video_dev *viddev, int width, int height, unsigned short input, unsigned short norm, unsigned long freq)
 {
 	int dev_bktr=viddev->fd_bktr;
 	//int dev_tunner=viddev->fd_tuner;
 	/* to ensure that all device will be support the capture mode 
 	  _TODO_ : Autodected the best capture mode .
 	*/
+	int dummy = 1;
+//	int pixelformat = BSD_VIDFMT_I420;
 	int single  = METEOR_CAP_SINGLE;
-	char *map;
 
-	if ( ((freq <= 0) && (input == IN_TV)) || ( set_input(cnt, viddev, input) == -1) ) {
-		motion_log(cnt, LOG_ERR, 1, "Frequency [%lu] Source input [%i]",freq,input);
-		return (NULL);
-	}
- 
+	void *map;
+
 	/* if we have choose the tuner is needed to setup the frequency */
-	if ( input == IN_TV ) {
-		if (set_freq(cnt, viddev, freq) == -1) {
+	if ( (viddev->tuner_device != NULL) && ( input == IN_TV ) ) {
+		if (!freq) {
+			motion_log(LOG_ERR, 1, "Not valid Frequency [%lu] for Source input [%i]", freq, input);
+			return (NULL);
+		}else if (set_freq(viddev, freq) == -1) {
+			motion_log(LOG_ERR, 1, "Frequency [%lu] Source input [%i]", freq, input);
 			return (NULL);
 		}
 	}
 	
 	/* FIXME if we set as input tuner , we need to set option for tuner not for bktr */
 
-	if ( set_input_format(cnt, viddev, norm) == -1 ) {
+	if ( set_input_format(viddev, norm) == -1 ) {
+		motion_log(LOG_ERR, 1, "set input format [%d]",norm);
 		return (NULL);
 	}
 
-	if (set_geometry(cnt, viddev, width, height, norm) == -1) {
+	if (set_geometry(viddev, width, height) == -1) {
+		motion_log(LOG_ERR, 1, "set geometry [%d]x[%d]",width, height);
 		return (NULL);
 	}
-
 /*
+	if (ioctl(dev_bktr, METEORSACTPIXFMT, &pixelformat) < 0 ){
+        	motion_log(LOG_ERR, 1, "set encoding method BSD_VIDFMT_I420"); 
+		return(NULL);
+        }
+
+
+	NEEDED !? FIXME 
+
 	if ( setup_pixelformat(viddev) == -1) {
 		return (NULL);
 	}
-
-	set_brightness(viddev,BRIGHT);
-	set_chroma(viddev,CHROMA);
-	set_contrast(viddev,CONTR);
 */
 
-	if (freq != 0) {
+	if (freq) {
+		if (cnt->conf.setup_mode)
+			motion_log(-1, 0, "Frequency set (no implemented yet");
 	/*
 	 TODO missing implementation
 		set_channelset(viddev);
 		set_channel(viddev);
-		if (set_freq (cnt, viddev, freq) == -1) {
+		if (set_freq (viddev, freq) == -1) {
 			return (NULL);
 		}
 	*/
@@ -529,11 +627,12 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
 
 	viddev->v4l_bufsize = (((width*height*3/2)) * sizeof(unsigned char *));
 	viddev->v4l_fmt = VIDEO_PALETTE_YUV420P;
+	
 
 	map = mmap((caddr_t)0,viddev->v4l_bufsize,PROT_READ|PROT_WRITE,MAP_SHARED, dev_bktr, (off_t)0);
 
-	if ((unsigned char *)-1 == (unsigned char *)map) {
-		motion_log(cnt, LOG_ERR, 1, "mmap failed");
+	if (map == MAP_FAILED){
+		motion_log(LOG_ERR, 1, "mmap failed");
 		return (NULL);
 	}
 
@@ -541,7 +640,7 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
 	if (0) {
 		viddev->v4l_maxbuffer=2;
 		viddev->v4l_buffers[0]=map;
-		viddev->v4l_buffers[1]=map+0; /* 0 is not valid just a test */
+		viddev->v4l_buffers[1]=(unsigned char *)map+0; /* 0 is not valid just a test */
 		//viddev->v4l_buffers[1]=map+vid_buf.offsets[1];
 	} else {
 		viddev->v4l_buffers[0]=map;
@@ -550,21 +649,25 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
 
 	viddev->v4l_curbuffer=0;
 
+	/* Clear the buffer */
+
+        if (ioctl(dev_bktr, BT848SCBUF, &dummy) < 0) {
+                motion_log(LOG_ERR, 1, "BT848SCBUF");
+                return NULL;
+        }
+
 	// settle , sleep(1) replaced
 	SLEEP(1,0)
 
-	/* capture */
 	if (ioctl(dev_bktr, METEORCAPTUR, &single) < 0){
-		motion_log(cnt, LOG_ERR, 1, "METEORCAPTUR Error capturing");
-	}
- 	
-	
-	/* Palete Color space */
+		motion_log(LOG_ERR, 1, "METEORCAPTUR using single method Error capturing");
+        }else viddev->capture_method = CAPTURE_SINGLE;
+
 	/* FIXME*/
 	switch (viddev->v4l_fmt) {
 		case VIDEO_PALETTE_YUV420P:
 			viddev->v4l_bufsize=(width*height*3)/2;
-			motion_log(cnt, -1, 0, "VIDEO_PALETTE_YUV420P palette setting bufsize");
+			motion_log(-1, 0, "VIDEO_PALETTE_YUV420P palette setting bufsize");
 			break;
 		case VIDEO_PALETTE_YUV422:
 			viddev->v4l_bufsize=(width*height*2);
@@ -576,7 +679,14 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
 			viddev->v4l_bufsize=width*height;
 			break;
 	}
-
+	
+	{
+	int val;
+	motion_log(LOG_INFO,0,"HUE [%d]",get_hue(dev_bktr,&val));
+	motion_log(LOG_INFO,0,"SATURATION [%d]",get_saturation(dev_bktr,&val));
+	motion_log(LOG_INFO,0,"BRIGHTNESS [%d]",get_brightness(dev_bktr,&val));
+	motion_log(LOG_INFO,0,"CONTRAST [%d]",get_contrast(dev_bktr,&val));
+	}
 	return map;
 }
 
@@ -584,7 +694,6 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
 /**
  * v4l_next fetches a video frame from a v4l device
  * Parameters:
- *     cnt        Pointer to the context for this thread
  *     viddev     Pointer to struct containing video device handle
  *     map        Pointer to the buffer in which the function puts the new image
  *     width      Width of image in pixels
@@ -595,59 +704,64 @@ static char *v4l_start(struct context *cnt, struct video_dev *viddev, int width,
  *    -1          Fatal error
  *     1          Non fatal error (not implemented)
  */
-int v4l_next(struct context *cnt, struct video_dev *viddev, char *map, int width, int height)
+static int v4l_next(struct video_dev *viddev,unsigned char *map, int width, int height)
 {
 	int dev_bktr=viddev->fd_bktr;
-	char *cap_map=NULL;
+	unsigned char *cap_map=NULL;
 	int single  = METEOR_CAP_SINGLE;
+	int continous = METEOR_CAP_CONTINOUS;
 	sigset_t  set, old;
 
 
-	if (viddev->v4l_read_img) {
-	/* use read to get capture the image if mmap not supported */	
-		if (read(dev_bktr, map, viddev->v4l_bufsize) != viddev->v4l_bufsize) {
-			motion_log(cnt, LOG_ERR, 0, "capturing frame");
-			return -1;
-		}	
-	} else {
+	/* ONLY MMAP method is used to Capture */
+
 	/* Allocate a new mmap buffer */
-		/* Block signals during IOCTL */
-		sigemptyset (&set);
-		sigaddset (&set, SIGCHLD);
-		sigaddset (&set, SIGALRM);
-		sigaddset (&set, SIGUSR1);
-		sigaddset (&set, SIGTERM);
-		sigaddset (&set, SIGHUP);
-		pthread_sigmask (SIG_BLOCK, &set, &old);
-		cap_map=viddev->v4l_buffers[viddev->v4l_curbuffer];
+	/* Block signals during IOCTL */
+	sigemptyset (&set);
+	sigaddset (&set, SIGCHLD);
+	sigaddset (&set, SIGALRM);
+	sigaddset (&set, SIGUSR1);
+	sigaddset (&set, SIGTERM);
+	sigaddset (&set, SIGHUP);
+	pthread_sigmask (SIG_BLOCK, &set, &old);
+	cap_map=viddev->v4l_buffers[viddev->v4l_curbuffer];
 
-		viddev->v4l_curbuffer++;
-		if (viddev->v4l_curbuffer >= viddev->v4l_maxbuffer)
-			viddev->v4l_curbuffer=0;
+	viddev->v4l_curbuffer++;
+	if (viddev->v4l_curbuffer >= viddev->v4l_maxbuffer)
+		viddev->v4l_curbuffer=0;
 
-		/* capture */
-		if (ioctl(dev_bktr, METEORCAPTUR, &single) < 0) {
-			motion_log(cnt, LOG_ERR, 1, "Error capturing");
+	/* capture */
+
+	if (viddev->capture_method == CAPTURE_CONTINOUS){
+		if (ioctl(dev_bktr, METEORCAPTUR, &continous) < 0) {
+			motion_log(LOG_ERR, 1, "Error capturing using continuous method");
 			sigprocmask (SIG_UNBLOCK, &old, NULL);
 			return (-1);
 		}
+	}else{
 
-		pthread_sigmask (SIG_UNBLOCK, &old, NULL);	/*undo the signal blocking*/
-	}
-
-	if (!viddev->v4l_read_img) 
-	{
-		switch (viddev->v4l_fmt){
-			case VIDEO_PALETTE_RGB24:
-				rgb24toyuv420p(map, cap_map, width, height);
-				break;
-			case VIDEO_PALETTE_YUV422:
-				yuv422to420p(map, cap_map, width, height);
-				break;
-			default:
-				memcpy(map, cap_map, viddev->v4l_bufsize);
+		if (ioctl(dev_bktr, METEORCAPTUR, &single) < 0) {
+			motion_log(LOG_ERR, 1, "Error capturing using single method");
+			sigprocmask (SIG_UNBLOCK, &old, NULL);
+			return (-1);
 		}
 	}
+
+	/*undo the signal blocking*/
+	pthread_sigmask (SIG_UNBLOCK, &old, NULL);
+	
+
+	switch (viddev->v4l_fmt){
+		case VIDEO_PALETTE_RGB24:
+			rgb24toyuv420p(map, cap_map, width, height);
+			break;
+		case VIDEO_PALETTE_YUV422:
+			yuv422to420p(map, cap_map, width, height);
+			break;
+		default:
+			memcpy(map, cap_map, viddev->v4l_bufsize);
+	}
+
 	
 	return 0;
 }
@@ -655,60 +769,65 @@ int v4l_next(struct context *cnt, struct video_dev *viddev, char *map, int width
 
 /* set input & freq if needed FIXME not allowed use Tuner yet */
 
-void v4l_set_input(struct context *cnt, struct video_dev *viddev, char *map, int width, int height,
-                    int input, int norm, int skip, unsigned long freq)
+static void v4l_set_input(struct context *cnt, struct video_dev *viddev, unsigned char *map, int width, int height,
+                    unsigned short input, unsigned short norm, int skip, unsigned long freq)
 {
 	int i;
 	unsigned long frequnits = freq;
 
-
 	if (input != viddev->input || width != viddev->width || height!=viddev->height || freq!=viddev->freq){ 
-		motion_log(cnt, LOG_WARNING, 0, "v4l_set_input really needed ?");
+		motion_log(LOG_WARNING, 0, "v4l_set_input really needed ?");
 
-		if (set_input(cnt, viddev, input) == -1)
+		if (set_input(viddev, input) == -1)
 			return;
 
-		if (set_input_format(cnt, viddev, norm) == -1 )
+		if (set_input_format(viddev, norm) == -1 )
 			return;
 		
-		if (( input == IN_TV ) && (frequnits > 0)) {
-			if (set_freq (cnt, viddev, freq) == -1)
-				return ;
+		if ((viddev->tuner_device != NULL) && ( input == IN_TV ) && (frequnits > 0)) {
+			if (set_freq (viddev, freq) == -1)
+				return;
 		}
 
 		// FIXME
 		/*
 		if ( setup_pixelformat(viddev) == -1) {
-			motion_log(cnt, LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
-			return;
+			motion_log(LOG_ERR, 1, "ioctl (VIDIOCSFREQ)");
+			return
 		}
 		*/
 
-		if (set_geometry(cnt, viddev, width, height, norm) == -1)
+		if (set_geometry(viddev, width, height) == -1)
 			return;
+	
+		v4l_picture_controls(cnt, viddev);
 		
 		viddev->input=input;
 		viddev->width=width;
 		viddev->height=height;
 		viddev->freq=freq;
+		viddev->norm=norm;
 
 		/* skip a few frames if needed */
 		for (i=0; i<skip; i++)
-			v4l_next(cnt, viddev, map, width, height);
+			v4l_next(viddev, map, width, height);
+	}else{
+		/* No round robin - we only adjust picture controls */
+		v4l_picture_controls(cnt, viddev);
 	}
 }
 
 /*****************************************************************************
-	Wrappers calling the actual capture routines
+	Wrappers calling the current capture routines
  *****************************************************************************/
 
 /*
 
-vid_init - Allocate viddev struct
-vid_start - Setup Device parameters ( device , channel , freq , contrast , hue , croma , brightness ) and open it
-vid_next - 
-vid_close - close devices 
-vid_cleanup - Free viddev struct
+vid_init - Allocate viddev struct.
+vid_start - Setup Device parameters ( device , channel , freq , contrast , hue , saturation , brightness ) and open it.
+vid_next - Capture a frame and set input , contrast , hue , saturation and brightness if necessary. 
+vid_close - close devices. 
+vid_cleanup - Free viddev struct.
 
 */
 
@@ -717,10 +836,10 @@ pthread_mutex_t vid_mutex;
 /* structure used for per device locking */
 struct video_dev **viddevs=NULL;
 
-void vid_init(struct context *cnt)
+void vid_init(void)
 {
 	if (!viddevs) { 
-		viddevs=mymalloc(cnt, sizeof(struct video_dev *));
+		viddevs=mymalloc(sizeof(struct video_dev *));
 		viddevs[0]=NULL;
 	}
 
@@ -731,15 +850,9 @@ void vid_init(struct context *cnt)
 void vid_close(void)
 {
 	int i=-1;
-//	int stopcap=0;
 
 	if (viddevs){ 
 		while(viddevs[++i]){
-		/*
-			stopcap = METEOR_CAP_STOP_CONT;
-			if( ioctl( viddevs[++i]->fd_bktr, METEORCAPTUR, &stopcap ) < 0 )
-				motion_log(cnt,LOG_ERR , 1, "vid_close METEORCAPTUR");	
-		*/
 			close(viddevs[i]->fd_bktr);
 			close(viddevs[i]->fd_tuner);
 		}
@@ -752,6 +865,7 @@ void vid_cleanup(void)
 	if (viddevs){ 
 		while(viddevs[++i]){
 			munmap(viddevs[i]->v4l_buffers[0],viddevs[i]->v4l_bufsize);
+			viddevs[i]->v4l_buffers[0] = MAP_FAILED;
 			free(viddevs[i]);
 		}
 		free(viddevs);
@@ -765,21 +879,36 @@ void vid_cleanup(void)
 int vid_start(struct context *cnt)
 {
 	struct config *conf=&cnt->conf;
-	int fd_bktr=-1,fd_tuner=-1;
+	int fd_bktr=-1;
 	
 	if (conf->netcam_url)
 		return netcam_start(cnt);
 
 #ifndef WITHOUT_V4L
 	{
+		int fd_tuner=-1;
 		int i=-1;
-		int width, height, input, norm;
+		int width, height;
+		unsigned short input, norm;
 		unsigned long frequency;
 
 		/* We use width and height from conf in this function. They will be assigned
 		 * to width and height in imgs here, and cap_width and cap_height in 
 		 * rotate_data won't be set until in rotate_init.
+		 * Motion requires that width and height are multiples of 16 so we check for this
 		 */
+		if (conf->width % 16) {
+			motion_log(LOG_ERR, 0,
+			           "config image width (%d) is not modulo 16",
+			           conf->width);
+			return -1;
+		}
+		if (conf->height % 16) {
+			motion_log(LOG_ERR, 0,
+			           "config image height (%d) is not modulo 16",
+			           conf->height);
+			return -1;
+		}
 		width = conf->width;
 		height = conf->height;
 		input = conf->input;
@@ -790,16 +919,21 @@ int vid_start(struct context *cnt)
 
 		/* Transfer width and height from conf to imgs. The imgs values are the ones
 		 * that is used internally in Motion. That way, setting width and height via
-		 * xmlrpc won't screw things up.
+		 * http remote control won't screw things up.
 		 */
 		cnt->imgs.width=width;
 		cnt->imgs.height=height;
 
+		/* First we walk through the already discovered video devices to see
+		 * if we have already setup the same device before. If this is the case
+		 * the device is a Round Robin device and we set the basic settings
+		 * and return the file descriptor
+		 */
 		while (viddevs[++i]) { 
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
 				int fd;
 				cnt->imgs.type=viddevs[i]->v4l_fmt;
-				motion_log(cnt, -1, 0, "vid_start cnt->imgs.type [%i]", cnt->imgs.type);
+				motion_log(-1, 0, "vid_start cnt->imgs.type [%i]", cnt->imgs.type);
 				switch (cnt->imgs.type) {
 					case VIDEO_PALETTE_GREY:
 						cnt->imgs.motionsize=width*height;
@@ -809,7 +943,8 @@ int vid_start(struct context *cnt)
 					case VIDEO_PALETTE_YUV422:
 						cnt->imgs.type=VIDEO_PALETTE_YUV420P;
 					case VIDEO_PALETTE_YUV420P:
-						motion_log(cnt, -1, 0, " VIDEO_PALETTE_YUV420P setting imgs.size and imgs.motionsize");
+						motion_log(-1, 0,
+						           " VIDEO_PALETTE_YUV420P setting imgs.size and imgs.motionsize");
 						cnt->imgs.motionsize=width*height;
 						cnt->imgs.size=(width*height*3)/2;
 					break;
@@ -820,8 +955,8 @@ int vid_start(struct context *cnt)
 			}
 		}
 
-		viddevs=myrealloc(cnt, viddevs, sizeof(struct video_dev *)*(i+2), "vid_start");
-		viddevs[i]=mymalloc(cnt, sizeof(struct video_dev));
+		viddevs=myrealloc(viddevs, sizeof(struct video_dev *)*(i+2), "vid_start");
+		viddevs[i]=mymalloc(sizeof(struct video_dev));
 		viddevs[i+1]=NULL;
 
 		pthread_mutexattr_init(&viddevs[i]->attr);
@@ -829,18 +964,18 @@ int vid_start(struct context *cnt)
 
 		fd_bktr=open(conf->video_device, O_RDWR);
 		if (fd_bktr <0) { 
-			motion_log(cnt, LOG_ERR, 1, "open video device %s",conf->video_device);
-			motion_log(cnt, LOG_ERR, 0, "Motion Exits.");
-			exit(1);
+			motion_log(LOG_ERR, 1, "open video device %s",conf->video_device);
+			motion_log(LOG_ERR, 0, "Motion Exits.");
+			return -1;
 		}
 
-		/* Only open tuner if freq > 0 FIXME ?! */
-		if (( frequency > 0 ) && ( input = IN_TV )) {
+		/* Only open tuner if conf->tuner_device has set , freq and input is 1 */
+		if ( (conf->tuner_device != NULL) && (frequency > 0) && ( input == IN_TV )) {
 			fd_tuner=open(conf->tuner_device, O_RDWR);
-			if (fd_bktr <0) { 
-				motion_log(cnt, LOG_ERR, 1, "open tuner device %s",conf->tuner_device);
-				motion_log(cnt, LOG_ERR, 0, "Motion Exits.");
-				exit(1);
+			if (fd_tuner <0) { 
+				motion_log(LOG_ERR, 1, "open tuner device %s",conf->tuner_device);
+				motion_log(LOG_ERR, 0, "Motion Exits.");
+				return -1;
 			}
 		}
 
@@ -854,9 +989,18 @@ int vid_start(struct context *cnt)
 		viddevs[i]->freq=frequency;
 		viddevs[i]->owner=-1;
 
+		/* We set brightness, contrast, saturation and hue = 0 so that they only get
+                 * set if the config is not zero.
+                 */
+		
+		viddevs[i]->brightness=0;
+		viddevs[i]->contrast=0;
+		viddevs[i]->saturation=0;
+		viddevs[i]->hue=0;
+		viddevs[i]->owner=-1;
+
 		/* default palette */ 
 		viddevs[i]->v4l_fmt=VIDEO_PALETTE_YUV420P;
-		viddevs[i]->v4l_read_img=0;
 		viddevs[i]->v4l_curbuffer=0;
 		viddevs[i]->v4l_maxbuffer=1;
 
@@ -876,7 +1020,7 @@ int vid_start(struct context *cnt)
 			case VIDEO_PALETTE_YUV422:
 				cnt->imgs.type=VIDEO_PALETTE_YUV420P;
 			case VIDEO_PALETTE_YUV420P:
-				motion_log(cnt, -1, 0, "VIDEO_PALETTE_YUV420P imgs.type");
+				motion_log(-1, 0, "VIDEO_PALETTE_YUV420P imgs.type");
 				cnt->imgs.size=(width*height*3)/2;
 				cnt->imgs.motionsize=width*height;
 			break;
@@ -904,12 +1048,15 @@ int vid_start(struct context *cnt)
  *     1          Non fatal V4L error (not implemented)
  *     2          Non fatal Netcam error
  */
-int vid_next(struct context *cnt, char *map)
+int vid_next(struct context *cnt, unsigned char *map)
 {
 	struct config *conf=&cnt->conf;
 	int ret = -1;
 
 	if (conf->netcam_url) {
+		if (cnt->video_dev == -1)
+			return NETCAM_GENERAL_ERROR;
+
 		ret = netcam_next(cnt, map);
 		return ret;
 	}
@@ -935,14 +1082,14 @@ int vid_next(struct context *cnt, char *map)
 		pthread_mutex_lock(&viddevs[i]->mutex);
 		viddevs[i]->owner=cnt->threadnr;
 		viddevs[i]->frames=conf->roundrobin_frames;
-		//viddevs[i]->frames=1;
 		cnt->switched=1;
 	}
 
 
 	v4l_set_input(cnt, viddevs[i], map, width, height, conf->input, conf->norm,
 	              conf->roundrobin_skip, conf->frequency);
-	ret = v4l_next(cnt, viddevs[i], map, width, height);
+	
+	ret = v4l_next(viddevs[i], map, width, height);
 
 	if (--viddevs[i]->frames <= 0) { 
 		viddevs[i]->owner=-1;
