@@ -12,8 +12,8 @@
 
 #ifndef WITHOUT_V4L
 
-#include <machine/ioctl_meteor.h>
-#include <machine/ioctl_bt848.h>
+#include <dev/bktr/ioctl_meteor.h>
+#include <dev/bktr/ioctl_bt848.h>
 
 #endif
 
@@ -30,33 +30,58 @@
 #define TV_INPUT_PALNCOMB BT848_IFORM_F_RSVD
 */
 
+/* video4linux error codes */
+#define V4L_GENERAL_ERROR    0x01          /* binary 000001 */
+#define V4L_BTTVLOST_ERROR   0x05          /* binary 000101 */
+#define V4L_FATAL_ERROR      -1
+
 #define NORM_DEFAULT	0x00800 // METEOR_FMT_AUTOMODE 
 #define NORM_PAL	0x00200 // METEOR_FMT_PAL 
 #define NORM_NTSC	0x00100 // METEOR_FMT_NTSC 
 #define NORM_SECAM	0x00400 // METEOR_FMT_SECAM
 #define NORM_PAL_NC	0x00200 // METEOR_FMT_PAL /* Greyscale howto ?! FIXME */
 
+#define NORM_DEFAULT_NEW	 BT848_IFORM_F_AUTO 
+#define NORM_PAL_NEW		 BT848_IFORM_F_PALBDGHI	
+#define NORM_NTSC_NEW		 BT848_IFORM_F_NTSCM
+#define NORM_SECAM_NEW		 BT848_IFORM_F_SECAM	         	
+#define NORM_PAL_NC_NEW		 BT848_IFORM_F_AUTO /* FIXME */	
+
+#define BSD_VIDFMT_NONE       0
+#define BSD_VIDFMT_YV12       1
+#define BSD_VIDFMT_I420       2
+#define BSD_VIDFMT_YV16       3
+#define BSD_VIDFMT_YUY2       4
+#define BSD_VIDFMT_UYVY       5
+#define BSD_VIDFMT_RV15       6
+#define BSD_VIDFMT_RV16       7
+#define BSD_VIDFMT_LAST       8
+
+
 #define IN_DEFAULT      0 
 #define IN_COMPOSITE    0
 #define IN_TV           1
 #define IN_COMPOSITE2   2
 #define IN_SVIDEO       3
 
+#define CAPTURE_SINGLE    0
+#define CAPTURE_CONTINOUS 1
 
 #define VIDEO_DEVICE "/dev/bktr0"
-#define TUNER_DEVICE "/dev/turner0"
-
+#define TUNER_DEVICE "/dev/tuner0"
 
 struct video_dev {
 	int fd_bktr;
 	int fd_tuner;
-	char *video_device;
-	char *tuner_device;
-	int input;
+	const char *video_device;
+	const char *tuner_device;
+	unsigned short input;
+	unsigned short norm;
 	int width;
 	int height;
 	int contrast;
-	int chroma;
+	int saturation;
+	int hue;
 	int brightness;
 	int channel;
 	int channelset;
@@ -69,9 +94,8 @@ struct video_dev {
 	
 	/* Device type specific stuff: */
 #ifndef WITHOUT_V4L	
-	/* v4l */
+ 	int capture_method;	
 	int v4l_fmt;
-	int v4l_read_img;
 	unsigned char *v4l_buffers[2];
 	int v4l_curbuffer;
 	int v4l_maxbuffer;
@@ -81,16 +105,12 @@ struct video_dev {
 
 /* video functions, video_freebsd.c */
 int vid_start(struct context *);
-int vid_next(struct context *, char *map);
+int vid_next(struct context *, unsigned char *map);
 
 #ifndef WITHOUT_V4L
-void vid_init(struct context *);
+void vid_init(void);
 void vid_close(void);
 void vid_cleanup(void);
 #endif
 
-/* Network camera functions, netcam.c */
-int netcam_start(struct context *);
-int netcam_next(struct context *, unsigned char *);
-
 #endif /* _INCLUDE_VIDEO_FREEBSD_H */
